@@ -33,13 +33,14 @@ namespace EncToolsBRC
 
     constexpr mfxU8  LA_P_UPDATE_DIST = 8;
     constexpr mfxU8  MBQP_P_UPDATE_DIST = 7;
-    constexpr mfxU8  MAX_GOP_REFDIST = 8;
+    constexpr mfxU8  MAX_GOP_REFDIST = 16;
     constexpr mfxU8  MIN_PAQ_QP = 10;   //do not allow PAQ to set QP below this value
 
     constexpr mfxF64 MIN_RACA = 0.25;
     constexpr mfxF64 MAX_RACA = 361.0;
     constexpr mfxF64 RACA_SCALE = 128.0;
 
+    constexpr mfxU8 TOTAL_NUM_AV1_SEGMENTS_FOR_ENCTOOLS = 7;
 /*
 NalHrdConformance | VuiNalHrdParameters   |  Result
 --------------------------------------------------------------
@@ -59,6 +60,7 @@ struct BRC_FrameStruct
     mfxU32 encOrder         = 0;
     mfxU32 dispOrder        = 0;
     mfxI32 qp               = 0;
+    mfxI32 origSeqQp        = 0;
     mfxI32 frameSize        = 0;
     mfxI32 numRecode        = 0;
     mfxU16 sceneChange      = 0;
@@ -72,7 +74,7 @@ struct BRC_FrameStruct
     mfxU16 miniGopSize      = 0;
     mfxU16 QpMapNZ          = 0;
     mfxU16 PersistenceMapNZ = 0;
-
+    mfxI16 QpMapBias        = 0;
     mfxU8  PersistenceMap[MFX_ENCTOOLS_PREENC_MAP_SIZE] = {};
 };
 
@@ -148,6 +150,7 @@ public:
     mfxU16   mHasALTR; // When mHasALTR, LTR marking decision (on/off) based on content.
     mfxU32   codecId;
     bool     mMBBRC;    // Enable Macroblock-CU level QP control (true/false)
+    bool     lowPower;
 
 public:
     cBRCParams() :
@@ -203,7 +206,8 @@ public:
         mLaScale(0),
         mHasALTR(0),
         codecId(0),
-        mMBBRC(false)
+        mMBBRC(false),
+        lowPower(0)
     {}
 
     mfxStatus Init(mfxEncToolsCtrl const & ctrl, bool bMBBRC, bool bFieldMode);
@@ -303,6 +307,70 @@ private:
 
 };
 
+struct LA_Ctx
+{
+LA_Ctx():
+    LastLaPBitsAvg(LA_P_UPDATE_DIST + MAX_GOP_REFDIST, 0),
+    LastLaQpCalc(0),
+    LastLaQpCalcEncOrder(0),
+    LastLaQpCalcDispOrder(0),
+    LastLaQpCalcHasI(0),
+    LastLaQpUpdateEncOrder(0),
+    LastLaQpUpdateDispOrder(0),
+    LastLaIBits(0),
+    LastMBQpSetEncOrder(0),
+    LastMBQpSetDispOrder(0)
+    {}
+
+    std::vector<mfxU32> LastLaPBitsAvg;  // History of Moving Avg of LA bits of last P frames
+    mfxU32 LastLaQpCalc;                 // Last LaQp calculated
+    mfxU32 LastLaQpCalcEncOrder;
+    mfxU32 LastLaQpCalcDispOrder;
+    bool   LastLaQpCalcHasI;
+    mfxU32 LastLaQpUpdateEncOrder;
+    mfxU32 LastLaQpUpdateDispOrder;
+    mfxU32 LastLaIBits;                 // LA bits of Last Intra Frame
+    mfxU32 LastMBQpSetEncOrder;
+    mfxU32 LastMBQpSetDispOrder;
+
+    bool IsCalcLaQpDist(mfxU32 dispOrder)
+    {
+        return dispOrder >= LastLaQpCalcDispOrder + LA_P_UPDATE_DIST;
+    }
+    bool IsUpdateLaQpDist(mfxU32 dispOrder)
+    {
+        return dispOrder >= LastLaQpUpdateDispOrder + LA_P_UPDATE_DIST;
+    }
+    void SetLaQpCalcOrder(mfxU32 encOrder, mfxU32 dispOrder)
+    {
+        LastLaQpCalcEncOrder = encOrder;
+        LastLaQpCalcDispOrder = dispOrder;
+    }
+    void SetLaQpUpdateOrder(mfxU32 encOrder, mfxU32 dispOrder) 
+    { 
+        LastLaQpUpdateEncOrder = encOrder;
+        LastLaQpUpdateDispOrder = dispOrder;
+    }
+    mfxU32 GetLastCalcLaBitsAvg(mfxU32 encOrder)
+    {
+        return LastLaPBitsAvg[mfx::clamp((mfxI32)encOrder - (mfxI32)LastLaQpCalcEncOrder - 1, 0, (mfxI32)LastLaPBitsAvg.size() - 1)];
+    }
+    void SaveLaBits(bool intra, mfxU32 LaCurEncodedSize, mfxU32 period, bool rotate)
+    {
+        if (intra) LastLaIBits = LaCurEncodedSize;
+        if (rotate) std::rotate(LastLaPBitsAvg.rbegin(), LastLaPBitsAvg.rbegin() + 1, LastLaPBitsAvg.rend());
+        if (LastLaPBitsAvg[1])
+        {
+            LastLaPBitsAvg[0] = (mfxI32)LastLaPBitsAvg[1] +
+                ((mfxI32)LaCurEncodedSize - (mfxI32)LastLaPBitsAvg[1]) / (mfxI32)period;
+        }
+        else
+        {
+            LastLaPBitsAvg[0] = LaCurEncodedSize;
+        }
+    }
+};
+
 struct BRC_Ctx
 {
     BRC_Ctx() :
@@ -325,17 +393,12 @@ struct BRC_Ctx
         LastIQpAct(0),
         LastIFrameSize(0),
         LastICmplx(0),
-        LastLaIBits(0),
         LastIQpSetOrder(0),
         LastQpUpdateOrder(0),
         LastIQpMin(0),
         LastIQpSet(0),
-        LastLaPBitsAvg(LA_P_UPDATE_DIST + MAX_GOP_REFDIST, 0),
-        LastLaQpCalc(0),
-        LastLaQpCalcOrder(0),
-        LastMBQpSetOrder(0),
-        LastLaQpUpdateOrder(0),
         LastNonBFrameSize(0),
+        la(),
         fAbLong(0),
         fAbShort(0),
         fAbLA(0),
@@ -366,19 +429,14 @@ struct BRC_Ctx
     mfxU32 LastIQpAct;      // Qp of last intra frame
     mfxU32 LastIFrameSize; // encoded frame size of last non B frame (is used for sceneChange)
     mfxF64 LastICmplx;      // Cmplx of last intra frame
-    mfxU32 LastLaIBits;     // La bits of Last Intra Frame
+
     mfxU32 LastIQpSetOrder; // Qp of last intra frame
     mfxU32 LastQpUpdateOrder;   // When using UpdateQpParams
     mfxU32 LastIQpMin;      // Qp of last intra frame
     mfxU32 LastIQpSet;      // Qp of last intra frame
-    std::vector<mfxU32> LastLaPBitsAvg;  // History of Moving Avg of LA bits of last P frames
-    mfxU32 LastLaQpCalc;   // Last LaQp calculated
-    mfxU32 LastLaQpCalcOrder;
-    mfxU32 LastMBQpSetOrder;
-    mfxU32 LastLaQpUpdateOrder;
 
     mfxU32 LastNonBFrameSize; // encoded frame size of last non B frame (is used for sceneChange)
-
+    LA_Ctx la;
     mfxF64 fAbLong;         // frame aberration (long period)
     mfxF64 fAbShort;        // frame aberration (short period)
     mfxF64 fAbLA;           // frame aberration (LA period)
@@ -554,20 +612,55 @@ protected:
     mfxU32     m_ReEncodeCount;
     std::vector<BRC_FrameStruct> m_FrameStruct;
 
-    virtual mfxU16 FillQpMap(const BRC_FrameStruct&, mfxU32 /*frameQp*/, mfxEncToolsHintQPMap*) = 0;
+    virtual mfxU16 FillQpMap(const BRC_FrameStruct&, mfxU32 /*frameQp*/, mfxEncToolsHintQPMap*, mfxI16&) = 0;
 
     mfxI32 GetCurQP(mfxU32 type, mfxI32 layer, mfxU16 isRef, mfxU16 qpMod, mfxI32 qpDeltaP) const;
     mfxI32 GetSeqQP(mfxI32 qp, mfxU32 type, mfxI32 layer, mfxU16 isRef, mfxU16 qpMod, mfxI32 qpDeltaP) const;
     mfxI32 GetPicQP(mfxI32 qp, mfxU32 type, mfxI32 layer, mfxU16 isRef, mfxU16 qpMod, mfxI32 qpDeltaP) const;
     mfxF64 ResetQuantAb(mfxI32 qp, mfxU32 type, mfxI32 layer, mfxU16 isRef, mfxF64 fAbLong, mfxU32 eo, bool bIdr, mfxU16 qpMod, mfxI32 qpDeltaP, bool bNoNewQp) const;
-    mfxI32 GetLaQpEst(mfxU32 LaAvgEncodedSize, mfxF64 inputBitsPerFrame) const;
+    mfxI32 GetLaQpEst(mfxU32 LaAvgEncodedSize, mfxF64 inputBitsPerFrame, const BRC_FrameStruct& frameStruct, bool updateState);
 };
 
 class BRC_EncTool : public BRC_EncToolBase
 {
 public:
 
-    mfxU16 FillQpMap(const BRC_FrameStruct& frameStruct, mfxU32 frameQp, mfxEncToolsHintQPMap* qpMap) override;
+    mfxU16 FillQpMap(const BRC_FrameStruct& frameStruct, mfxU32 frameQp, mfxEncToolsHintQPMap* qpMap, mfxI16& qpMapBias) override;
+};
+
+/*
+    - Current AV1 enctools is based on HEVC enctools
+    - Following 2 LUTs are for the conversion of HEVC QP and AV1 dc_q_idx.  
+*/
+
+// HEVC QP to AV1 dc_q_index
+mfxU8 const HEVC_QP_2_AV1_DC_Q_IDX[52] = 
+{
+    1,   1,   1,   1,   2,   3,   4,   5,   7,   9,   11,  13,  16,  19,  21,  26, 
+    29,  34,  39,  45,  52,  59,  68,  78,  89,  98,  106, 114, 122, 130, 138, 145,
+    153, 162, 170, 178, 186, 195, 203, 211, 219, 226, 231, 236, 241, 245, 248, 251,
+    252, 253, 254, 255
+};
+
+// AV1 dc_q_index to HEVC QP
+mfxU8 const AV1_DC_Q_IDX_2_HEVC_QP[256] = 
+{
+    0, 4, 4, 5, 6, 7, 8, 8, 8, 9, 9, 10,11,11,11,11,
+    12,12,13,13,14,14,14,14,15,15,15,16,16,16,16,16,
+    17,17,17,17,17,17,17,18,18,18,18,19,19,19,19,19,
+    20,20,20,20,20,20,20,20,21,21,21,21,21,21,21,22,
+    22,22,22,22,22,22,22,22,22,23,23,23,23,23,23,23,
+    23,23,23,23,23,24,24,24,24,24,24,24,24,24,24,24,
+    25,25,25,25,25,25,26,26,26,26,26,26,26,26,27,27,
+    27,27,27,27,27,27,27,28,28,28,28,28,28,28,29,29,
+    29,29,29,29,29,29,29,30,30,30,30,30,30,30,31,31,
+    31,31,31,31,31,31,32,32,32,32,32,32,32,32,33,33,
+    33,33,33,33,33,33,34,34,34,34,34,34,34,34,35,35,
+    35,35,35,35,35,35,35,36,36,36,36,36,36,36,36,37,
+    37,37,37,37,37,37,37,38,38,38,38,38,38,38,38,38,
+    39,39,39,39,39,39,39,39,40,40,40,40,40,40,40,41,
+    41,41,41,41,41,42,42,42,42,42,43,43,43,43,43,44,
+    44,44,44,45,45,45,45,46,46,46,46,47,47,47,48,48
 };
 
 }
