@@ -38,12 +38,12 @@ mfxStatus InitCtrl(mfxVideoParam const & par, mfxEncToolsCtrl *ctrl)
     mfxExtCodingOptionDDI* extDdi = (mfxExtCodingOptionDDI*)Et_GetExtBuffer(par.ExtParam, par.NumExtParam, MFX_EXTBUFF_DDI);
     MFX_CHECK_NULL_PTR1(extDdi);
 
-    ctrl = {};
+    *ctrl = {};
 
     ctrl->CodecId = par.mfx.CodecId;
     ctrl->CodecProfile = par.mfx.CodecProfile;
     ctrl->CodecLevel = par.mfx.CodecLevel;
-
+    ctrl->LowPower = par.mfx.LowPower;
     ctrl->AsyncDepth = par.AsyncDepth;
 
     ctrl->FrameInfo = par.mfx.FrameInfo;
@@ -87,9 +87,9 @@ mfxStatus InitCtrl(mfxVideoParam const & par, mfxEncToolsCtrl *ctrl)
         else if (IsOn(CO->NalHrdConformance) && IsOff(CO->VuiNalHrdParameters))
             ctrl->HRDConformance = MFX_BRC_HRD_WEAK;
 
+        ctrl->BufferSizeInKB = par.mfx.BufferSizeInKB*mult;            //Bitstream size
         if (ctrl->HRDConformance)
         {
-            ctrl->BufferSizeInKB = par.mfx.BufferSizeInKB*mult;      //if HRDConformance is ON
             ctrl->InitialDelayInKB = par.mfx.InitialDelayInKB*mult;    //if HRDConformance is ON
         }
         else
@@ -253,7 +253,8 @@ mfxStatus EncTools::GetSupportedConfig(mfxExtEncToolsConfig* config, mfxEncTools
             ctrl->RateControlMethod == MFX_RATECONTROL_VBR) ?
             MFX_CODINGOPTION_ON : MFX_CODINGOPTION_OFF);
 
-        if (ctrl->MaxGopRefDist == 8 ||
+        if (ctrl->MaxGopRefDist == 16 ||
+            ctrl->MaxGopRefDist == 8 ||
             ctrl->MaxGopRefDist == 4 ||
             ctrl->MaxGopRefDist == 2 ||
             ctrl->MaxGopRefDist == 1)
@@ -267,6 +268,7 @@ mfxStatus EncTools::GetSupportedConfig(mfxExtEncToolsConfig* config, mfxEncTools
             config->AdaptivePyramidQuantP = MFX_CODINGOPTION_ON;
             config->AdaptivePyramidQuantB = MFX_CODINGOPTION_ON;
             config->AdaptiveMBQP = MFX_CODINGOPTION_ON;
+            config->AdaptiveQuantMatrices = MFX_CODINGOPTION_ON;
             if (ctrl->MaxDelayInFrames > ctrl->MaxGopRefDist && IsOn(config->BRC))
                 config->BRCBufferHints = MFX_CODINGOPTION_ON;
         }
@@ -572,6 +574,14 @@ mfxStatus EncTools::InitMfxVppParams(mfxEncToolsCtrl const & ctrl)
     if (!mfxVppParams_Common.vpp.In.CropH)
         mfxVppParams_Common.vpp.In.CropH = mfxVppParams_Common.vpp.In.Height;
 
+    if(ctrl.ScenarioInfo != MFX_SCENARIO_GAME_STREAMING && mfxVppParams_Common.vpp.Out.BitDepthLuma == 10){
+        mfxVppParams_Common.vpp.Out.FourCC = MFX_FOURCC_NV12;
+        mfxVppParams_Common.vpp.Out.ChromaFormat = MFX_CHROMAFORMAT_YUV420;
+        mfxVppParams_Common.vpp.Out.BitDepthLuma = 8;
+        mfxVppParams_Common.vpp.Out.BitDepthChroma = 8;
+        mfxVppParams_Common.vpp.Out.Shift = 0;
+    }
+
     MFXDLVideoSession* pSession = nullptr;
     if (isPreEncSCD(m_config, ctrl))
         pSession = &m_mfxSession_SCD;
@@ -624,7 +634,7 @@ mfxStatus EncTools::CloseVPP()
         m_pmfxVPP_LA->Close();
         m_pmfxVPP_LA.reset();
     }
-    
+
     m_mfxSession_LA = nullptr;
 
     if (m_IntSurfaces_SCD.Data.Y)
@@ -824,6 +834,7 @@ mfxStatus EncTools::Reset(mfxExtEncToolsConfig const * config, mfxEncToolsCtrl c
 mfxStatus EncTools::VPPDownScaleSurface(MFXDLVideoSession* /*m_pmfxSession*/, MFXDLVideoVPP* pVPP, mfxSyncPoint* pVppSyncp, mfxFrameSurface1* pInSurface, mfxFrameSurface1* pOutSurface/*, bool doSync*/)
 {
     mfxStatus sts;
+    MFX_CHECK_NULL_PTR2(pVPP, pVppSyncp);
     MFX_CHECK_NULL_PTR2(pInSurface, pOutSurface);
 
     sts = pVPP->RunFrameVPPAsync(pInSurface, pOutSurface, NULL, pVppSyncp);
@@ -836,6 +847,45 @@ static void IgnoreMoreDataStatus(mfxStatus &sts)
         sts = MFX_ERR_NONE;
 }
 
+// Compute AV1 segmentIDs and AltQIndexes
+// Note: Due to HW limitation, one segment must be reserved for HW use.
+//       PAQ assumes segment 7 is reserved for HW use.  This assumption imposes constraint on segMap file in commandline (ie,  segMap must has all numbers from 0 to 6).
+static mfxStatus ProcessAV1SegmentationData(const mfxEncToolsTaskParam* par, mfxEncToolsHintQPMap* qpMapHint, mfxI32 QpY)
+{
+    MFX_CHECK(par && qpMapHint, MFX_ERR_UNDEFINED_BEHAVIOR);
+
+    mfxExtAV1Segmentation* seg = (mfxExtAV1Segmentation *)Et_GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_AV1_SEGMENTATION);
+    MFX_CHECK(seg, MFX_ERR_UNDEFINED_BEHAVIOR);
+
+    // AV1 segment IDs
+    //  - QpY is HEVC QP for frame. qpMapHint.ExtQpMap.QP[] is HEVC QP for blocks/segments.
+    //  - Note SegmentID is derived from HEVC QP
+
+    mfxI16 segQPs[EncToolsBRC::TOTAL_NUM_AV1_SEGMENTS_FOR_ENCTOOLS] = {-1, -1, -1, -1, -1, -1, -1};  //"-1" means feature is disabled
+    mfxU8 maxQpY = *(std::max_element(qpMapHint->ExtQpMap.QP, qpMapHint->ExtQpMap.QP + qpMapHint->ExtQpMap.NumQPAlloc));
+    for(mfxU32 i = 0; i < qpMapHint->ExtQpMap.NumQPAlloc ; i++) {
+        mfxI16 seg_qp = (mfxI16)(qpMapHint->ExtQpMap.QP[i]);
+        mfxU8 id = mfxU8(mfx::clamp((mfxI16)maxQpY - seg_qp, 0, EncToolsBRC::TOTAL_NUM_AV1_SEGMENTS_FOR_ENCTOOLS - 1)); // PAQ uses segments [0..6]
+        segQPs[id] = seg_qp;
+        seg->SegmentIds[i] = id;
+    }
+
+    //Update new AltQIndex (feature_value)
+    //  - Convert from HEVC QP to AV1 q-idx
+    mfxI16 q_idx = mfxI16(EncToolsBRC::HEVC_QP_2_AV1_DC_Q_IDX[QpY]);
+    for (mfxI16 id = 0; id < EncToolsBRC::TOTAL_NUM_AV1_SEGMENTS_FOR_ENCTOOLS; id++) {
+        if (segQPs[id] != -1) {
+            seg->Segment[id].FeatureEnabled |= MFX_VP9_SEGMENT_FEATURE_QINDEX;
+            mfxI16 altQIndex = (mfxI16)EncToolsBRC::HEVC_QP_2_AV1_DC_Q_IDX[segQPs[id]] - q_idx;
+            seg->Segment[id].AltQIndex = mfx::clamp(altQIndex, (mfxI16)-255, (mfxI16)255);
+        } else {
+            seg->Segment[id].FeatureEnabled &= (~MFX_VP9_SEGMENT_FEATURE_QINDEX);
+        }
+    }
+
+    return MFX_ERR_NONE;
+}
+
 mfxStatus EncTools::Submit(mfxEncToolsTaskParam const * par)
 {
     mfxStatus sts = MFX_ERR_NONE;
@@ -961,6 +1011,12 @@ mfxStatus EncTools::Submit(mfxEncToolsTaskParam const * par)
 
 
     mfxEncToolsBRCEncodeResult  *pEncRes = (mfxEncToolsBRCEncodeResult *)Et_GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_ENCTOOLS_BRC_ENCODE_RESULT);
+
+    // Since AV1 BRC uses HEVC BRC, so need this conversion
+    if(pEncRes && m_ctrl.CodecId == MFX_CODEC_AV1) {
+        pEncRes->QpY = EncToolsBRC::AV1_DC_Q_IDX_2_HEVC_QP[pEncRes->QpY];        
+    }
+
     if (pEncRes && isPreEncSCD(m_config, m_ctrl)) {
         m_scd.ReportEncResult(par->DisplayOrder, *pEncRes);
     }
@@ -1056,10 +1112,27 @@ mfxStatus EncTools::Query(mfxEncToolsTaskParam* par, mfxU32 /*timeOut*/)
     mfxEncToolsBRCQuantControl *pFrameQp = (mfxEncToolsBRCQuantControl *)Et_GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_ENCTOOLS_BRC_QUANT_CONTROL);
     mfxEncToolsHintQPMap* qpMapHint = (mfxEncToolsHintQPMap*)Et_GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_ENCTOOLS_HINT_QPMAP);
 
+    // Sanity check
+    if (qpMapHint && m_ctrl.CodecId == MFX_CODEC_AV1) {
+        mfxU16 w = m_ctrl.FrameInfo.Width;
+        mfxU16 pitch = (w + qpMapHint->ExtQpMap.BlockSize - 1) / qpMapHint->ExtQpMap.BlockSize;
+        MFX_CHECK(qpMapHint->QpMapPitch == pitch, MFX_ERR_UNDEFINED_BEHAVIOR);
+    }
+
     if (pFrameQp && IsOn(m_config.BRC))
     {
         sts = m_brc->ProcessFrame(par->DisplayOrder, pFrameQp, qpMapHint);
         MFX_CHECK_STS(sts);
+
+        // Note: AV1 BRC is based on HEVC BRC, so need this conversion
+        if (m_ctrl.CodecId == MFX_CODEC_AV1) {
+            // Compute segmentIDs and AltQIndexes
+            if (qpMapHint && qpMapHint->QpMapFilled) {
+                ProcessAV1SegmentationData(par, qpMapHint, (mfxI32)(pFrameQp->QpY));
+            }
+
+            pFrameQp->QpY =  EncToolsBRC::HEVC_QP_2_AV1_DC_Q_IDX[pFrameQp->QpY];
+        }
     }
     mfxEncToolsBRCHRDPos *pHRDPos = (mfxEncToolsBRCHRDPos *)Et_GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_ENCTOOLS_BRC_HRD_POS);
     if (pHRDPos && IsOn(m_config.BRC))
