@@ -48,6 +48,11 @@
 #include "libmfx_core_interface.h"
 
 
+#include "mfx_unified_av1d_logging.h"
+#if defined(MFX_ENABLE_PXP)
+#include "umc_va_protected.h"
+#endif // MFX_ENABLE_PXP
+
 namespace MFX_VPX_Utility
 {
     inline
@@ -128,7 +133,8 @@ VideoDECODEAV1::VideoDECODEAV1(VideoCORE* core, mfxStatus* sts)
     , m_is_init(false)
     , m_in_framerate(0)
     , m_is_cscInUse(false)
-    , m_anchor_frames_num(0)
+    , m_anchorFramesSource(0)
+    , m_va(nullptr)
 {
     if (sts)
     {
@@ -268,7 +274,8 @@ mfxStatus VideoDECODEAV1::Init(mfxVideoParam* par)
     sts = m_core->CreateVA(par, &m_request, &m_response, m_surface_source.get());
     MFX_CHECK_STS(sts);
 
-    m_core->GetVA((mfxHDL*)&vp.pVideoAccelerator, MFX_MEMTYPE_FROM_DECODE);
+    m_core->GetVA((mfxHDL*)&m_va, MFX_MEMTYPE_FROM_DECODE);
+    vp.pVideoAccelerator = m_va;
 
     ConvertMFXParamsToUMC(par, &vp);
     vp.info.profile = av1_mfx_profile_to_native_profile(par->mfx.CodecProfile);
@@ -500,6 +507,7 @@ mfxStatus VideoDECODEAV1::Close()
     m_response_alien = {};
 
     m_is_init = false;
+    m_va = nullptr;
 
     return MFX_ERR_NONE;
 }
@@ -619,7 +627,6 @@ mfxStatus VideoDECODEAV1::QueryIOSurf(VideoCORE* core, mfxVideoParam* par, mfxFr
 
     request->Type |= MFX_MEMTYPE_EXTERNAL_FRAME;
 
-
     return sts;
 }
 
@@ -746,6 +753,15 @@ mfxStatus VideoDECODEAV1::DecodeFrameCheck(mfxBitstream* bs, mfxFrameSurface1* s
     MFX_CHECK(m_core, MFX_ERR_UNDEFINED_BEHAVIOR);
     MFX_CHECK(m_decoder, MFX_ERR_NOT_INITIALIZED);
 
+#if defined(MFX_ENABLE_PXP)
+    //Check protect VA is enabled or not
+    if( bs && m_va && m_va->GetProtectedVA())
+    {
+        MFX_CHECK((bs->DataFlag & MFX_BITSTREAM_COMPLETE_FRAME), MFX_ERR_UNSUPPORTED);
+        m_va->GetProtectedVA()->SetBitstream(bs);
+    }
+#endif // MFX_ENABLE_PXP
+
     mfxStatus sts = SubmitFrame(bs, surface_work, surface_out);
 
     if (sts == MFX_ERR_MORE_DATA || sts == MFX_ERR_MORE_SURFACE)
@@ -825,6 +841,10 @@ mfxStatus VideoDECODEAV1::DecodeFrame(mfxFrameSurface1 *surface_out, AV1DecoderF
     UMC::FrameMemID id = frame->GetFrameData()->GetFrameMID();
     mfxStatus sts = m_surface_source->PrepareToOutput(surface_out, id, &m_video_par);
     frame->Displayed(true);
+
+    TRACE_EVENT(MFX_TRACE_API_AV1_DISPLAYINFO_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_BASIC_INFO, make_event_data(
+        id, (uint32_t)frame->Displayed(), (uint32_t)frame->Outputted()));
+
     return sts;
 }
 
@@ -839,6 +859,9 @@ mfxStatus VideoDECODEAV1::QueryFrame(mfxThreadTask task)
         reinterpret_cast<TaskInfo*>(task);
     UMC_AV1_DECODER::AV1DecoderFrame* frame = NULL;
     mfxFrameSurface1* surface_out = info->surface_out;
+    MFX_CHECK_NULL_PTR1(surface_out);
+    MFX_CHECK(surface_out, MFX_ERR_INVALID_HANDLE);
+
     if(info->copyfromframe != UMC::FRAME_MID_INVALID)
     {
         frame = m_decoder->DecodeFrameID(info->copyfromframe);
@@ -923,7 +946,7 @@ mfxStatus VideoDECODEAV1::SubmitFrame(mfxBitstream* bs, mfxFrameSurface1* surfac
     }
 
     if (!bs)
-        return MFX_ERR_MORE_DATA;
+        MFX_RETURN(MFX_ERR_MORE_DATA);
 
     mfxStatus sts = CheckBitstream(bs);
     MFX_CHECK_STS(sts);
@@ -934,6 +957,10 @@ mfxStatus VideoDECODEAV1::SubmitFrame(mfxBitstream* bs, mfxFrameSurface1* surfac
 
         for (;;)
         {
+            if (!m_decoder->IsFreeSlotInDPB()){
+                return MFX_WRN_DEVICE_BUSY;
+            }
+
             sts = m_surface_source->SetCurrentMFXSurface(surface_work);
             MFX_CHECK_STS(sts);
 
@@ -1000,7 +1027,7 @@ mfxStatus VideoDECODEAV1::SubmitFrame(mfxBitstream* bs, mfxFrameSurface1* surfac
                     break;
 
                 default:
-                    if (sts < 0)
+                    if (sts < 0 || umcRes < 0)
                         sts = MFX_ERR_UNDEFINED_BEHAVIOR;
                     break;
             }
@@ -1064,16 +1091,28 @@ mfxStatus VideoDECODEAV1::SubmitFrame(mfxBitstream* bs, mfxFrameSurface1* surfac
 
 mfxStatus VideoDECODEAV1::DecodeRoutine(void* state, void* param, mfxU32, mfxU32)
 {
-    auto decoder = reinterpret_cast<VideoDECODEAV1*>(state);
-    MFX_CHECK(decoder, MFX_ERR_UNDEFINED_BEHAVIOR);
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_INTERNAL, "AV1DECODERoutine");
+    mfxStatus sts = MFX_ERR_NONE;
 
-    mfxThreadTask task =
-        reinterpret_cast<TaskInfo*>(param);
+    try
+    {
+        auto decoder = reinterpret_cast<VideoDECODEAV1*>(state);
+        MFX_CHECK(decoder, MFX_ERR_UNDEFINED_BEHAVIOR);
 
-    MFX_CHECK(task, MFX_ERR_UNDEFINED_BEHAVIOR);
+        mfxThreadTask task =
+            reinterpret_cast<TaskInfo*>(param);
+
+        MFX_CHECK(task, MFX_ERR_UNDEFINED_BEHAVIOR);
+
+        sts = decoder->QueryFrame(task);
+        MFX_CHECK_STS(sts);
+    }
+    catch(...)
+    {
+        MFX_LTRACE_MSG_1(MFX_TRACE_LEVEL_INTERNAL, "exception handled");
+        return MFX_ERR_NONE;
+    }
 
-    mfxStatus sts = decoder->QueryFrame(task);
-    MFX_CHECK_STS(sts);
     return sts;
 }
 
@@ -1282,8 +1321,8 @@ mfxStatus VideoDECODEAV1::FillOutputSurface(mfxFrameSurface1** surf_out, mfxFram
     surface_out->Data.FrameOrder = pFrame->FrameOrder();
     surface_out->Data.Corrupted = 0;
     surface_out->Info.PicStruct = MFX_PICSTRUCT_PROGRESSIVE;
-    surface_out->Info.CropW = static_cast<mfxU16>(m_anchor_frames_num > 0 ? pFrame->GetRenderWidth() : pFrame->GetUpscaledWidth());
-    surface_out->Info.CropH = static_cast<mfxU16>(m_anchor_frames_num > 0 ? pFrame->GetRenderHeight() : pFrame->GetFrameHeight());
+    surface_out->Info.CropW = static_cast<mfxU16>(m_anchorFramesSource > 0 ? pFrame->GetRenderWidth() : pFrame->GetUpscaledWidth());
+    surface_out->Info.CropH = static_cast<mfxU16>(m_anchorFramesSource > 0 ? pFrame->GetRenderHeight() : pFrame->GetFrameHeight());
     surface_out->Info.AspectRatioW = 1;
     surface_out->Info.AspectRatioH = 1;
 
@@ -1303,6 +1342,9 @@ mfxStatus VideoDECODEAV1::FillOutputSurface(mfxFrameSurface1** surf_out, mfxFram
     surface_out->Info.FrameRateExtD = isShouldUpdate ? m_init_par.mfx.FrameInfo.FrameRateExtD : m_first_par.mfx.FrameInfo.FrameRateExtD;
     surface_out->Info.FrameRateExtN = isShouldUpdate ? m_init_par.mfx.FrameInfo.FrameRateExtN : m_first_par.mfx.FrameInfo.FrameRateExtN;
 
+    TRACE_BUFFER_EVENT(MFX_TRACE_API_AV1_OUTPUTINFO_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_BASIC_INFO,
+        surface_out, AV1DecodeSurfaceOutparam, SURFACEOUT_AV1D);
+
     return MFX_ERR_NONE;
 }
 
