@@ -43,6 +43,8 @@
 #include "libmfx_core_interface.h"
 using namespace UMC_VP9_DECODER;
 
+#include "mfx_unified_vp9d_logging.h"
+
 static bool IsSameVideoParam(mfxVideoParam *newPar, mfxVideoParam *oldPar);
 
 // function checks hardware support with IsGuidSupported function and GUIDs
@@ -246,6 +248,12 @@ public:
         VP9_CHECK_AND_THROW((find_it == m_submittedFrames.end()), MFX_ERR_UNKNOWN);
 
         m_submittedFrames.push_back(frame);
+
+        if (!m_submittedFrames.empty())
+        {
+            TRACE_BUFFER_EVENT(MFX_TRACE_API_VP9_DPBPARAMETER_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_DPB_INFO,
+                m_submittedFrames, VP9DecodeDpbInfo, DPBINFO_VP9D);
+        }
     }
 
     void DecodeFrame(UMC::FrameMemID frameId)
@@ -257,6 +265,9 @@ public:
         {
             find_it->isDecoded = true;
         }
+
+        TRACE_EVENT(MFX_TRACE_API_VP9_DISPLAYINFO_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_BASIC_INFO, make_event_data(
+            find_it->currFrame, (uint32_t)find_it->isDecoded));
     }
 
     void CompleteFrames()
@@ -299,7 +310,8 @@ VideoDECODEVP9_HW::VideoDECODEVP9_HW(VideoCORE *p_core, mfxStatus *sts)
       m_completedList(),
       m_firstSizes(),
       m_bs(),
-      m_baseQIndex(0)
+      m_baseQIndex(0),
+      m_skipParseSuperFrameIndex(false)
 {
     memset(&m_sizesOfRefFrame, 0, sizeof(m_sizesOfRefFrame));
     memset(&m_frameInfo.ref_frame_map, VP9_INVALID_REF_FRAME, sizeof(m_frameInfo.ref_frame_map)); // move to another place
@@ -809,7 +821,7 @@ mfxStatus VideoDECODEVP9_HW::GetUserData(mfxU8 *pUserData, mfxU32 *pSize, mfxU64
 
     MFX_CHECK_NULL_PTR3(pUserData, pSize, pTimeStamp);
 
-    return MFX_ERR_UNSUPPORTED;
+    MFX_RETURN(MFX_ERR_UNSUPPORTED);
 }
 
 mfxStatus VideoDECODEVP9_HW::GetPayload(mfxU64 *pTimeStamp, mfxPayload *pPayload)
@@ -820,7 +832,7 @@ mfxStatus VideoDECODEVP9_HW::GetPayload(mfxU64 *pTimeStamp, mfxPayload *pPayload
 
     MFX_CHECK_NULL_PTR3(pTimeStamp, pPayload, pPayload->Data);
 
-    return MFX_ERR_UNSUPPORTED;
+    MFX_RETURN(MFX_ERR_UNSUPPORTED);
 }
 
 mfxStatus VideoDECODEVP9_HW::SetSkipMode(mfxSkipMode /*mode*/)
@@ -859,6 +871,9 @@ mfxStatus MFX_CDECL VP9DECODERoutine(void *p_state, void * /* pp_param */, mfxU3
     VideoDECODEVP9_HW::VP9DECODERoutineData& data = *(VideoDECODEVP9_HW::VP9DECODERoutineData*)p_state;
     VideoDECODEVP9_HW& decoder = *data.decoder;
 
+    TRACE_EVENT(MFX_TRACE_API_VP9_SYNCINFO_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_BASIC_INFO, make_event_data(
+        data.copyFromFrame, data.currFrameId, data.index, data.showFrame));
+
     if (data.copyFromFrame != UMC::FRAME_MID_INVALID)
     {
         MFX_CHECK(data.surface_work, MFX_ERR_UNDEFINED_BEHAVIOR);
@@ -885,7 +900,11 @@ mfxStatus MFX_CDECL VP9DECODERoutine(void *p_state, void * /* pp_param */, mfxU3
             }
 
             UMC::AutomaticUMCMutex guardCopy(decoder.m_mCopyGuard[data.copyFromFrame]);
-            MFX_SAFE_CALL(decoder.m_core->DoFastCopyWrapper(&surfaceDst, dstMemType, surfaceSrc, srcMemType, decoder.m_core->GetVAType() == MFX_HW_VAAPI));
+            mfxU32 useGPUCopy = decoder.m_core->GetVAType() == MFX_HW_VAAPI ? MFX_COPY_USE_CM : 0u;
+            // not using CM copy in video to video copy becasue there are issues in CM copy.
+            if (!systemMemory)
+                useGPUCopy = 0;
+            MFX_SAFE_CALL(decoder.m_core->DoFastCopyWrapper(&surfaceDst, dstMemType, surfaceSrc, srcMemType, useGPUCopy));
         }
 
         if (data.currFrameId != -1)
@@ -1067,10 +1086,10 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
     }
 
 #if defined(MFX_ENABLE_PXP)
-	//Check protect VA is enabled or not
+    //Check protect VA is enabled or not
     if( bs && m_va && m_va->GetProtectedVA())
     {
-        MFX_CHECK(m_va->GetProtectedVA(), MFX_ERR_UNSUPPORTED);
+        m_skipParseSuperFrameIndex = true;
         MFX_CHECK((bs->DataFlag & MFX_BITSTREAM_COMPLETE_FRAME), MFX_ERR_UNSUPPORTED);
         m_va->GetProtectedVA()->SetBitstream(bs);
     }
@@ -1086,14 +1105,14 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
         {
             UMC::Status status = e.GetStatus();
             if (status != UMC::UMC_OK)
-                return MFX_ERR_UNKNOWN;
+                MFX_RETURN(MFX_ERR_UNKNOWN);
         }
 
         if (NeedToReturnCriticalStatus(bs))
             return ReturningCriticalStatus();
 
         if (bs && !bs->DataLength)
-            return MFX_ERR_MORE_DATA;
+            MFX_RETURN(MFX_ERR_MORE_DATA);
 
         if (bs == nullptr)
         {
@@ -1102,7 +1121,7 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
 
             if (m_framesStorage->IsAllFramesCompleted())
             {
-                return MFX_ERR_MORE_DATA;
+                MFX_RETURN(MFX_ERR_MORE_DATA);
             }
 
             return MFX_WRN_DEVICE_BUSY;
@@ -1132,7 +1151,7 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
                 m_vPar.mfx.FrameInfo.Height > surface_work->Info.Height)
             {
                 MFX_CHECK(m_adaptiveMode, MFX_ERR_INCOMPATIBLE_VIDEO_PARAM);
-                return MFX_ERR_REALLOC_SURFACE;
+                MFX_RETURN(MFX_ERR_REALLOC_SURFACE);
             }
         }
 
@@ -1169,9 +1188,13 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
         if (!m_frameInfo.show_existing_frame)
         {
             MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_HOTSPOTS, "VP9 decode DDISubmitTask");
-            PERF_EVENT(MFX_TRACE_HOTSPOT_DDI_SUBMIT_TASK, 0, make_event_data(this), [&]() { return make_event_data(sts);});
+
+            TRACE_EVENT(MFX_TRACE_HOTSPOT_DDI_SUBMIT_TASK, EVENT_TYPE_START, TR_KEY_DDI_API, make_event_data(++FrameIndex, m_frameInfo.currFrame));
 
             UMC::Status umcSts = m_va->BeginFrame(m_frameInfo.currFrame, 0);
+
+            TRACE_EVENT(MFX_TRACE_HOTSPOT_DDI_SUBMIT_TASK, EVENT_TYPE_END, TR_KEY_DDI_API, make_event_data(FrameIndex, m_frameInfo.currFrame, umcSts));
+
             MFX_CHECK(UMC::UMC_OK == umcSts, MFX_ERR_DEVICE_FAILED);
 
             sts = PackHeaders(&m_bs, m_frameInfo);
@@ -1180,8 +1203,12 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
             umcSts = m_va->Execute();
             MFX_CHECK(UMC::UMC_OK == umcSts, MFX_ERR_DEVICE_FAILED);
 
+            TRACE_EVENT(MFX_TRACE_HOTSPOT_DDI_ENDFRAME_TASK, EVENT_TYPE_START, TR_KEY_DDI_API, make_event_data(FrameIndex, umcSts));
 
             umcSts = m_va->EndFrame();
+
+            TRACE_EVENT(MFX_TRACE_HOTSPOT_DDI_ENDFRAME_TASK, EVENT_TYPE_END, TR_KEY_DDI_API, make_event_data(FrameIndex, sts));
+
             MFX_CHECK(UMC::UMC_OK == umcSts, MFX_ERR_DEVICE_FAILED);
         }
         else
@@ -1232,6 +1259,9 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
 
             m_frameOrder++;
             sts = MFX_ERR_NONE;
+
+            TRACE_BUFFER_EVENT(MFX_TRACE_API_VP9_OUTPUTINFO_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_BASIC_INFO,
+                surface_out, VP9DecodeSurfaceOutparam, SURFACEOUT_VP9D);
         }
         else
         {
@@ -1280,8 +1310,10 @@ mfxStatus VideoDECODEVP9_HW::DecodeSuperFrame(mfxBitstream *in, VP9DecoderFrame
     mfxU32 frameCount = 0;
 
     m_bs = *in;
-
-    MfxVP9Decode::ParseSuperFrameIndex(in->Data + in->DataOffset, in->DataLength, frameSizes, &frameCount);
+    if (!m_skipParseSuperFrameIndex)
+    {
+        MfxVP9Decode::ParseSuperFrameIndex(in->Data + in->DataOffset, in->DataLength, frameSizes, &frameCount);
+    }
 
     if (frameCount > 1)
     {
@@ -1323,7 +1355,7 @@ mfxStatus VideoDECODEVP9_HW::DecodeFrameHeader(mfxBitstream *in, VP9DecoderFrame
         VP9Bitstream bsReader(in->Data + in->DataOffset, in->DataOffset + in->DataLength);
 
         if (VP9_FRAME_MARKER != bsReader.GetBits(2))
-            return MFX_ERR_UNDEFINED_BEHAVIOR;
+            MFX_RETURN(MFX_ERR_UNDEFINED_BEHAVIOR);
 
         info.profile = bsReader.GetBit();
         info.profile |= bsReader.GetBit() << 1;
