@@ -27,6 +27,7 @@
 #include "av1ehw_ddi.h"
 #include "ehw_device.h"
 #include <vector>
+#include <set>
 
 namespace AV1EHW
 {
@@ -80,10 +81,58 @@ namespace Base
     const uint32_t AV1_MAX_NUM_TILE_COLS = 64;
 
     const uint16_t DEFAULT_TARGET_USAGE  = MFX_TARGETUSAGE_BALANCED;
+    const uint16_t DEFAULT_GOP_PIC_SIZE  = 248;
+    const uint16_t DEFAULT_GOP_REF_DIST  = 8;
     const mfxU8    IDX_INVALID           = 0xff;
 
     const int16_t  DEFAULT_BPYR_QP_OFFSETS[8] = {32, 40, 48, 48, 48, 48, 48, 48};
 
+    static const uint8_t LoopFilterLevelsLuma[256] = {
+        0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+        0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+        0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+        0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  1,  1,  2,
+        2,  2,  2,  2,  2,  2,  3,  3,  3,  3,  3,  3,  3,  4,  4,  4,
+        4,  4,  4,  4,  5,  5,  5,  5,  5,  5,  5,  6,  6,  6,  6,  6,
+        6,  7,  7,  7,  8,  8,  8,  8,  9,  9,  9,  9, 10, 10, 10, 10,
+        11, 11, 11, 11, 12, 12, 12, 12, 13, 13, 13, 14, 14, 14, 15, 15,
+        15, 16, 16, 16, 17, 17, 17, 17, 18, 18, 18, 19, 19, 20, 20, 20,
+        21, 21, 21, 22, 22, 22, 23, 23, 24, 24, 24, 25, 25, 25, 26, 26,
+        27, 27, 27, 28, 28, 29, 29, 29, 30, 30, 31, 31, 31, 32, 32, 33,
+        33, 34, 34, 34, 35, 35, 36, 36, 37, 37, 38, 38, 39, 39, 40, 41,
+        41, 42, 42, 43, 44, 45, 45, 46, 47, 48, 49, 50, 51, 52, 53, 55,
+        56, 58, 59, 61, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
+        63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63,
+        63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63, 63
+    };
+
+    static const uint8_t LoopFilterLevelsChroma[256] = {
+        0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+        0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+        0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+        0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  1,  1,  1,  1,  2,
+        2,  2,  2,  2,  2,  2,  2,  2,  2,  3,  3,  3,  3,  3,  3,  3,
+        3,  3,  3,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,
+        5,  5,  5,  5,  5,  5,  5,  5,  6,  6,  6,  6,  6,  6,  6,  6,
+        6,  6,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  8,  8,
+        8,  8,  8,  8,  8,  8,  8,  8,  9,  9,  9,  9,  9,  9,  9, 10,
+        10, 10, 10, 10, 11, 11, 11, 11, 12, 12, 13, 13, 14, 14, 15, 15,
+        16, 17, 18, 19, 20, 21, 22, 24, 25, 26, 28, 30, 31, 31, 31, 31,
+        31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
+        31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
+        31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
+        31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
+        31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31
+    };
+
+    static const uint8_t ICQFactorLookup[52] = { 
+        0,  1,  1,  1,  1,  2,  3,  4,  6,  7,  9,  11, 13, 16, 18, 23,
+        26, 30, 34, 39, 46, 52, 59, 68, 77, 87, 98, 104,112,120,127,134,
+        142,149,157,164,171,178,186,193,200,207,213,219,225,230,235,240,
+        245,249,252,255 
+    };
+
+
     inline bool IsValid(mfxU8 idx)
     {
         return idx != IDX_INVALID;
@@ -305,6 +354,8 @@ namespace Base
         uint32_t FeatureMask[AV1_MAX_NUM_OF_SEGMENTS];
     };
 
+    using SegDpbType = std::vector<std::shared_ptr<mfxExtAV1Segmentation>>;
+
     struct LoopFilterParams
     {
         int32_t loop_filter_level[4];
@@ -670,7 +721,6 @@ namespace Base
         mfxU32   EncodedOrder   = mfxU32(-1);
         bool     isLTR          = false; // is "long-term"
         bool     isRejected     = false; // rejected ref frame should be refreshed asap
-        bool     wasShown       = false;
         mfxU8    CodingType     = 0;
         Resource Raw;
         Resource Rec;
@@ -703,8 +753,7 @@ namespace Base
         , INSERT_TD             = (1 << 2)
         , INSERT_SPS            = (1 << 3)
         , INSERT_PPS            = (1 << 4)
-        , INSERT_REPEATED       = (1 << 5)
-        , INSERT_FRM_OBU        = (1 << 6)
+        , INSERT_FRM_OBU        = (1 << 5)
     };
 
     using DpbType = std::vector<std::shared_ptr<DpbFrame>>;
@@ -721,58 +770,91 @@ namespace Base
         mfxU8  FrameToShowMapIdx = 0;
         mfxU32 DisplayOrder      = 0;
     };
+
     struct mfxGopHints {
         mfxU32              MiniGopSize = 0;
         mfxU16              FrameType   = 0;
+        mfxU32              QPModulaton = 0;
+        /* Scene Change parameter */
+        mfxU16              SceneChange = 0;
+#if defined(MFX_ENABLE_ENCTOOLS)
+        mfxU16              SpatialComplexity = 0;
+        /* Persistence Map */
+        mfxU16              PersistenceMapNZ;
+        mfxU8               PersistenceMap[MFX_ENCTOOLS_PREENC_MAP_SIZE];
+#endif
     };
 
-    using RepeatedFrames = std::vector<RepeatedFrameInfo>;
-
-    using TileSizeType = decltype(mfxExtAV1AuxData::TileWidthInSB[0]);
+    using RepeatedFrames    = std::vector<RepeatedFrameInfo>;
+    using TileSizeType      = decltype(mfxExtAV1AuxData::TileWidthInSB[0]);
     using TileSizeArrayType = decltype(mfxExtAV1AuxData::TileWidthInSB);
 
+    struct mfxBRCHints {
+        /* Look ahead parameters */
+        mfxU32              LaAvgEncodedBits = 0;         /* Average size of encoded Lookahead frames in bits */
+        mfxU32              LaCurEncodedBits = 0;         /* Size of encoded Lookahead frame at current frame location in bits */
+        mfxU16              LaDistToNextI    = 0;         /* First I Frame in Lookahead frames (0 if not found) */
+    };
+
     struct TaskCommonPar
         : DpbFrame
     {
-        mfxU32            stage                = 0;
-        mfxU32            BsDataLength         = 0;
-        mfxU32            BsBytesAvailable     = 0;
-        mfxU8*            pBsData              = nullptr;
-        mfxU32*           pBsDataLength        = nullptr;
-        mfxBitstream*     pBsOut               = nullptr;
-        mfxFrameSurface1* pSurfReal            = nullptr;
-        mfxEncodeCtrl     ctrl                 = {};
-        mfxU32            SkipCMD              = SKIPCMD_NeedDriverCall;
+        mfxU32            stage                  = 0;
+        mfxU32            MinFrameSize           = 0;
+        mfxU32            BsDataLength           = 0;
+        mfxU32            BsBytesAvailable       = 0;
+        mfxU8*            pBsData                = nullptr;
+        mfxU32*           pBsDataLength          = nullptr;
+        mfxBitstream*     pBsOut                 = nullptr;
+        mfxFrameSurface1* pSurfReal              = nullptr;
+        mfxEncodeCtrl     ctrl                   = {};
+        mfxU32            SkipCMD                = SKIPCMD_NeedDriverCall;
         Resource          BS;
         Resource          CUQP;
-        mfxHDLPair        HDLRaw               = {};
-        bool              bCUQPMap             = false;
-        mfxGopHints       GopHints             = {};
-        bool              bForceSync           = false;
-        bool              bSkip                = false;
-        bool              bResetBRC            = false;
-        bool              bRecode              = false;
-        mfxI32            PrevRAP              = -1;
-        mfxU16            NumRecode            = 0;
-        mfxU8             QpY                  = 0;
-        mfxU32            InsertHeaders        = 0;
-        mfxU32            StatusReportId       = mfxU32(-1);
-        DpbRefreshType    RefreshFrameFlags    = {};
-        BitOffsets        Offsets              = {};
-        mfxU8             MinBaseQIndex        = 0;
-        mfxU8             MaxBaseQIndex        = 0;
+        mfxHDLPair        HDLRaw                 = {};
+        bool              bCUQPMap               = false;
+#if defined(MFX_ENABLE_ENCTOOLS)
+        mfxLplastatus     LplaStatus           = {};
+        mfxBRCHints      BrcHints             = {};
+        bool             bBRCUpdated          = false;
+#endif
+        mfxGopHints       GopHints               = {};
+        bool              bForceSync             = false;
+        bool              bSkip                  = false;
+        bool              bResetBRC              = false;
+        bool              bRecode                = false;
+        bool              bCachedTask            = false;
+        mfxI32            PrevRAP                = -1;
+        mfxU16            NumRecode              = 0;
+        mfxU8             QpY                    = 0;
+        mfxU32            InsertHeaders          = 0;
+        mfxU32            StatusReportId         = mfxU32(-1);
+        DpbRefreshType    RefreshFrameFlags      = {};
+        BitOffsets        Offsets                = {};
+        mfxU8             MinBaseQIndex          = 0;
+        mfxU8             MaxBaseQIndex          = 0;
 
-        RefListType       RefList              = {};
+        RefListType       RefList                = {};
         DpbType           DPB;
-        RepeatedFrames    FramesToShow         = {};
-        mfxU16            RepeatedFrameBytes   = 0;
-        mfxU16            PrevRepeatedFrameBytes = 0;
+        RepeatedFrames    FramesToShow           = {};
+        mfxU16            RepeatedFrameBytes     = 0;
+        mfxU32            TCBRCTargetFrameSize   = 0;
+
+        mfxExtRefListCtrl InternalListCtrl       = {};
+        bool              InternalListCtrlPresent = false;
     };
 
     inline void Invalidate(TaskCommonPar& par)
     {
         par = TaskCommonPar();
     }
+
+    inline bool IsHiddenFrame(const TaskCommonPar& frame)
+    {
+        return frame.NextBufferedDisplayOrder != -1 &&
+            frame.DisplayOrderInGOP > frame.NextBufferedDisplayOrder;
+    }
+
     inline bool isValid(DpbFrame const & frame) { return IDX_INVALID != frame.Rec.Idx; }
 
     inline std::tuple<mfxU32, mfxU32> GetRealResolution(const mfxVideoParam& vp)
@@ -854,6 +936,20 @@ namespace Base
         return std::make_tuple(numTileCols, numTileRows);
     }
 
+    inline mfxU32 GetAvgFrameSizeInBytes(const mfxVideoParam& par)
+    {
+        mfxU32 avgFrameSizeInBytes = 0;
+
+        if (par.mfx.FrameInfo.FrameRateExtN && par.mfx.FrameInfo.FrameRateExtD)
+        {
+            // Use type mfxU64 to avoid value overflow with most value precise obtained.
+            mfxU64 targetbps    = TargetKbps(par.mfx) * 1000;
+            avgFrameSizeInBytes = mfxU32(targetbps * par.mfx.FrameInfo.FrameRateExtD / par.mfx.FrameInfo.FrameRateExtN / 8);
+        }
+
+        return avgFrameSizeInBytes;
+    }
+
     class FrameLocker
         : public mfxFrameData
     {
@@ -1200,6 +1296,8 @@ namespace Base
         , FEATURE_ENCODED_FRAME_INFO
         , FEATURE_QMATRIX
         , FEATURE_QUERY_IMPL_DESC
+        , FEATURE_ENCTOOLS
+        , FEATURE_MAX_FRAME_SIZE
         , NUM_FEATURES
     };
 
@@ -1213,6 +1311,8 @@ namespace Base
         , NUM_STAGES
     };
 
+    using TFramesToShowInfo    = std::set<mfxU32>;
+    using TRepeatFrameSizeInfo = std::map<mfxU32, mfxU32>;
     struct Glob
     {
         static const StorageR::TKey _KD = __LINE__ + 1;
@@ -1222,6 +1322,8 @@ namespace Base
         using VideoParam          = StorageVar<__LINE__ - _KD, ExtBuffer::Param<mfxVideoParam>>;
         using SH                  = StorageVar<__LINE__ - _KD, Base::SH>;
         using FH                  = StorageVar<__LINE__ - _KD, Base::FH>;
+        using FramesToShowInfo    = StorageVar<__LINE__ - _KD, TFramesToShowInfo>;
+        using RepeatFrameSizeInfo = StorageVar<__LINE__ - _KD, TRepeatFrameSizeInfo>;
         using TileGroups          = StorageVar<__LINE__ - _KD, TileGroupInfos>;
         using AllocRaw            = StorageVar<__LINE__ - _KD, IAllocation>;
         using AllocRec            = StorageVar<__LINE__ - _KD, IAllocation>;
@@ -1235,6 +1337,7 @@ namespace Base
         using ResetHint           = StorageVar<__LINE__ - _KD, Base::ResetHint>; //available during Reset
         using Reorder             = StorageVar<__LINE__ - _KD, Reorderer>;
         using Defaults            = StorageVar<__LINE__ - _KD, Base::Defaults>;
+        using SegDpb              = StorageVar<__LINE__ - _KD, SegDpbType>;
         static const StorageR::TKey CallChainsKey = __LINE__ - _KD;
         static const StorageR::TKey ReservedKey0    = __LINE__ - _KD;
         static const StorageR::TKey TaskManagerKey  = __LINE__ - _KD;
@@ -1266,6 +1369,14 @@ namespace Base
         static const StorageR::TKey NUM_KEYS = __LINE__ - _KD;
     };
 
+    inline Defaults::Param GetRTDefaults(StorageR& strg)
+    {
+        return Defaults::Param(
+            Glob::VideoParam::Get(strg)
+            , Glob::EncodeCaps::Get(strg)
+            , Glob::Defaults::Get(strg));
+    }
+
 } //namespace Base
 } //namespace AV1EHW
 
