@@ -86,7 +86,12 @@ void InitSPS(
     sps.seq_fields.bits.enable_restoration         = bs_sh.enable_restoration;
 
     const mfxExtCodingOption2& CO2 = ExtBuffer::Get(par);
+#if VA_CHECK_VERSION(1, 16, 0)
+    sps.hierarchical_flag = CO2.BRefType == MFX_B_REF_PYRAMID;
+#else
     sps.reserved8b = CO2.BRefType == MFX_B_REF_PYRAMID;
+#endif
+
 }
 
 void InitPPS(
@@ -346,6 +351,20 @@ static void InitSegMap(
     InitSegMap(segPar, segment_map);
 }
 
+inline uint16_t MapLRTypeToDDI(RestorationType lrType)
+{
+    switch (lrType)
+    {
+    case RESTORE_NONE:
+        return 0;
+    case RESTORE_WIENER:
+        return 1;
+    default:
+        ThrowAssert(true, "Only RESTORE_NONE and RESTORE_WIENER are supported");
+        return 0;
+    }
+}
+
 void UpdatePPS(
     const TaskCommonPar& task
     , const std::vector<VASurfaceID>& rec
@@ -394,9 +413,9 @@ void UpdatePPS(
     FillCDEF(bs_sh, bs_fh, pps);
 
     //loop restoration
-    pps.loop_restoration_flags.bits.yframe_restoration_type  = bs_fh.lr_params.lr_type[0];
-    pps.loop_restoration_flags.bits.cbframe_restoration_type = bs_fh.lr_params.lr_type[1];
-    pps.loop_restoration_flags.bits.crframe_restoration_type = bs_fh.lr_params.lr_type[2];
+    pps.loop_restoration_flags.bits.yframe_restoration_type  = MapLRTypeToDDI(bs_fh.lr_params.lr_type[0]);
+    pps.loop_restoration_flags.bits.cbframe_restoration_type = MapLRTypeToDDI(bs_fh.lr_params.lr_type[1]);
+    pps.loop_restoration_flags.bits.crframe_restoration_type = MapLRTypeToDDI(bs_fh.lr_params.lr_type[2]);
     pps.loop_restoration_flags.bits.lr_unit_shift            = bs_fh.lr_params.lr_unit_shift;
     pps.loop_restoration_flags.bits.lr_uv_shift              = bs_fh.lr_params.lr_uv_shift;
 
@@ -437,7 +456,13 @@ void UpdatePPS(
     pps.qmatrix_flags.bits.qm_u = bs_fh.quantization_params.qm_u;
     pps.qmatrix_flags.bits.qm_v = bs_fh.quantization_params.qm_v;
 
+#if VA_CHECK_VERSION(1, 16, 0)
+    pps.hierarchical_level_plus1 = static_cast<mfxU8>(task.PyramidLevel + 1);
+#else
     pps.reserved8bits0 = static_cast<mfxU8>(task.PyramidLevel + 1);
+#endif
+
+    pps.skip_frames_reduced_size = static_cast<mfxI32>(task.RepeatedFrameBytes);
 }
 
 inline void AddVaMiscHRD(
