@@ -82,7 +82,7 @@ namespace Base
         num_NALU_TYPE
     };
 
-    struct PTL
+    struct ProfileTierLevel
     {
         mfxU8  profile_space : 2;
         mfxU8  tier_flag     : 1;
@@ -173,8 +173,8 @@ namespace Base
     struct LayersInfo
     {
         mfxU8 sub_layer_ordering_info_present_flag : 1;
-        PTL general;
-        struct SubLayer : PTL, SubLayerOrdering {} sub_layer[8];
+        ProfileTierLevel general;
+        struct SubLayer : ProfileTierLevel, SubLayerOrdering {} sub_layer[8];
     };
 
 
@@ -753,9 +753,12 @@ namespace Base
         mfxU32              QPModulaton = 0;
         /* Scene Change parameter */
         mfxU16              SceneChange = 0;
+#if defined(MFX_ENABLE_ENCTOOLS)
+        mfxU16              SpatialComplexity = 0;
         /* Maps */
         mfxU16              PersistenceMapNZ;
         mfxU8               PersistenceMap[128];
+#endif
     };
 
     struct mfxBRCHints {
@@ -1328,6 +1331,9 @@ namespace Base
         , FEATURE_BRC_SLIDING_WINDOW
         , FEATURE_MB_PER_SEC
         , FEATURE_ENCODED_FRAME_INFO
+#ifdef MFX_ENABLE_ENCODE_STATS
+        , FEATURE_ENCODE_STATS
+#endif
         , FEATURE_WEIGHTPRED
         , FEATURE_QMATRIX
         , FEATURE_UNITS_INFO
@@ -1344,6 +1350,9 @@ namespace Base
         , FEATURE_SCC
         , FEATURE_SCCMODE
         , FEATURE_CAPS
+#ifdef MFX_ENABLE_PXP
+        , FEATURE_PXP
+#endif
         , NUM_FEATURES
     };
 
@@ -1393,7 +1402,7 @@ namespace Base
         using RealState           = StorageVar<__LINE__ - _KD, StorageW>;//available during Reset
         using ResetHint           = StorageVar<__LINE__ - _KD, Base::ResetHint>; //available during Reset
         using Reorder             = StorageVar<__LINE__ - _KD, Reorderer>;
-        using NeedRextConstraints = StorageVar<__LINE__ - _KD, std::function<bool(const Base::PTL&)>>;
+        using NeedRextConstraints = StorageVar<__LINE__ - _KD, std::function<bool(const Base::ProfileTierLevel&)>>;
         using ReadSpsExt          = StorageVar<__LINE__ - _KD, std::function<bool(const Base::SPS&, mfxU8, IBsReader&)>>;
         using ReadPpsExt          = StorageVar<__LINE__ - _KD, std::function<bool(const Base::PPS&, mfxU8, IBsReader&)>>;
         using PackSpsExt          = StorageVar<__LINE__ - _KD, std::function<bool(const Base::SPS&, mfxU8, IBsWriter&)>>;
