@@ -210,7 +210,8 @@ inline bool  CheckSWEncCondition(const mfxVideoParam &video)
             || video.mfx.GopRefDist == 1
             || video.mfx.GopRefDist == 2
             || video.mfx.GopRefDist == 4
-            || video.mfx.GopRefDist == 8));
+            || video.mfx.GopRefDist == 8
+            || video.mfx.GopRefDist == 16));
 }
 
 inline mfxEncTools *GetEncTools(mfxVideoParam &video)
@@ -232,7 +233,7 @@ inline bool IsEncToolsImplicit(const mfxVideoParam &video)
         const mfxExtCodingOption3* pExtOpt3 = ExtBuffer::Get(video);
         etOn = (IsGameStreaming(video) && IsOn(video.mfx.LowPower)); // LPLA
         etOn = etOn ||
-            ((video.mfx.GopRefDist == 2 || video.mfx.GopRefDist == 8) && IsOn(pExtOpt2->ExtBRC) // SW EncTools
+            ((video.mfx.GopRefDist == 2 || video.mfx.GopRefDist == 4 || video.mfx.GopRefDist == 8 || video.mfx.GopRefDist == 16) && IsOn(pExtOpt2->ExtBRC) // SW EncTools
                 && !(pExtOpt3 && pExtOpt3->ScenarioInfo != MFX_SCENARIO_UNKNOWN));
     }
     return etOn;
@@ -311,7 +312,7 @@ static void SetDefaultConfig(const mfxVideoParam &video, mfxExtEncToolsConfig &c
         bool lplaAssistedBRC = IsOn(config.BRC) && isSWLACondition(video);
         SetDefaultOpt(config.BRCBufferHints, lplaAssistedBRC);
         SetDefaultOpt(config.AdaptiveMBQP,  bMBQPSupport && lplaAssistedBRC && IsOn(pExtOpt2->MBBRC));
-
+        SetDefaultOpt(config.AdaptiveQuantMatrices, false);
     }
 #ifdef MFX_ENABLE_ENCTOOLS_LPLA
     else
@@ -442,7 +443,7 @@ static mfxStatus InitEncToolsCtrl(
     ctrl->CodecId = par.mfx.CodecId;
     ctrl->CodecProfile = par.mfx.CodecProfile;
     ctrl->CodecLevel = par.mfx.CodecLevel;
-
+    ctrl->LowPower = par.mfx.LowPower;
     ctrl->AsyncDepth = par.AsyncDepth;
 
     ctrl->FrameInfo = par.mfx.FrameInfo;
@@ -492,9 +493,9 @@ static mfxStatus InitEncToolsCtrl(
         else
             ctrl->HRDConformance = MFX_BRC_HRD_STRONG;
 
+        ctrl->BufferSizeInKB   = par.mfx.BufferSizeInKB*mult;          //Bitstream size
         if (ctrl->HRDConformance)
         {
-            ctrl->BufferSizeInKB   = par.mfx.BufferSizeInKB*mult;      //if HRDConformance is ON
             ctrl->InitialDelayInKB = par.mfx.InitialDelayInKB*mult;    //if HRDConformance is ON
         }
         else
@@ -830,6 +831,7 @@ mfxStatus HevcEncTools::BRCGetCtrl(StorageW& global , StorageW& s_task,
         extFrameData.FrameType = task.FrameType;
         extFrameData.PyramidLayer = (mfxU16) task.PyramidLevel;
         extFrameData.SceneChange = task.GopHints.SceneChange;
+        extFrameData.SpatialComplexity = task.GopHints.SpatialComplexity;
         extFrameData.PersistenceMapNZ = task.GopHints.PersistenceMapNZ;
         memcpy(extFrameData.PersistenceMap, task.GopHints.PersistenceMap, sizeof(extFrameData.PersistenceMap));
 
@@ -1009,8 +1011,8 @@ mfxStatus HevcEncTools::QueryPreEncTask(StorageW&  global, StorageW& s_task)
     task.GopHints.MiniGopSize = preEncodeGOP.MiniGopSize;
     task.GopHints.FrameType = preEncodeGOP.FrameType;
     task.GopHints.SceneChange = preEncodeSChg.SceneChangeFlag;
+    task.GopHints.SpatialComplexity = preEncodeSChg.SpatialComplexity;
     task.GopHints.PersistenceMapNZ = preEncodeSChg.PersistenceMapNZ;
-
     if (qpMapHint.QpMapFilled)
         task.bCUQPMap = true;
     memcpy(task.GopHints.PersistenceMap, preEncodeSChg.PersistenceMap, sizeof(task.GopHints.PersistenceMap));
