@@ -1492,16 +1492,12 @@ mfxStatus CheckParameters(VP9MfxVideoParam &par, ENCODE_CAPS_VP9 const &caps)
         }
     }
 
-    mfxU16 numPipes = static_cast<mfxU16>(caps.NumScalablePipesMinus1 + 1);
-
-    if ((cols > numPipes && rows > 1) ||
-        (rows * cols > MAX_NUM_TILES))
+    if (rows * cols > MAX_NUM_TILES)
     {
         cols = 0;
         rows = 0;
         unsupported = true;
     }
-
     // KNOWN FEATURES LIMITATIONS:
 
     // temporal scalability and tiles don't work together
@@ -1566,7 +1562,7 @@ inline mfxU32 GetDefaultBufferSize(VP9MfxVideoParam const &par)
 
 inline mfxU16 GetDefaultAsyncDepth(VP9MfxVideoParam const &/*par*/)
 {
-    return 2;
+    return 5;
 }
 
 inline mfxU16 GetMinProfile(mfxU16 depth, mfxU16 format)
