@@ -39,8 +39,11 @@
 #include "libmfx_core_interface.h"
 #include "mfx_platform_caps.h"
 
+#include "mfx_unified_decode_logging.h"
+
 mfxStatus MFXInit(mfxIMPL implParam, mfxVersion *ver, mfxSession *session)
 {
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
     mfxInitParam par = {};
 
     par.Implementation = implParam;
@@ -73,13 +76,12 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
     mfxIMPL impl = par.Implementation & (MFX_IMPL_VIA_ANY - 1);
     mfxIMPL implInterface = par.Implementation & -MFX_IMPL_VIA_ANY;
 
-    MFXTrace_PerfInit();
     MFX_TRACE_INIT();
     {
         MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "ThreadName=MSDK app");
     }
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXInitEx");
-    PERF_EVENT(MFX_TRACE_API_MFX_INIT_EX_TASK, 0, make_event_data((mfxU32) par.Implementation, par.GPUCopy), [&](){ return make_event_data(mfxRes, session ? *session : nullptr);});
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
+    TRACE_EVENT(MFX_TRACE_API_MFX_INIT_EX_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data((mfxU32)par.Implementation, par.GPUCopy));
 
     // check the library version
     if (MakeVersion(par.Version.Major, par.Version.Minor) > MFX_VERSION)
@@ -150,6 +152,8 @@ mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
         (*session)->m_versionToReport.Minor = 255;
     }
 
+    TRACE_EVENT(MFX_TRACE_API_MFX_INIT_EX_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes, session));
+
     return mfxRes;
 
 } // mfxStatus MFXInitEx(mfxInitParam par, mfxSession *session)
@@ -199,8 +203,8 @@ mfxStatus MFXDoWork(mfxSession session)
 {
     mfxStatus res;
 
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXDoWork");
-    PERF_EVENT(MFX_TRACE_API_DO_WORK_TASK, 0, make_event_data(session), [&](){ return make_event_data(res);});
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
+    TRACE_EVENT(MFX_TRACE_API_DO_WORK_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data(session));
 
     // check error(s)
     if (0 == session)
@@ -220,22 +224,27 @@ mfxStatus MFXDoWork(mfxSession session)
 
     res = newScheduler->DoWork();
 
+    TRACE_EVENT(MFX_TRACE_API_DO_WORK_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(res));
+
     return res;
 } // mfxStatus MFXDoWork(mfxSession *session)
 
-struct MFXTrace_PerfCloseOnExit
+struct MFXTrace_EventCloseOnExit
 {
-    ~MFXTrace_PerfCloseOnExit()
+    ~MFXTrace_EventCloseOnExit()
     {
-        MFXTrace_PerfClose();
+        MFXTrace_EventClose();
     }
 };
 
 mfxStatus MFXClose(mfxSession session)
 {
     mfxStatus mfxRes = MFX_ERR_NONE;
-    MFXTrace_PerfCloseOnExit MFXTrace_PerfClose;
-
+    MFXTrace_EventCloseOnExit MFXTrace_EventClose;
+    if (EnableEventTrace)
+    {
+        TRACE_EVENT(MFX_TRACE_API_MFX_CLOSE_TASK, EVENT_TYPE_START, 0, make_event_data(session));
+    }
     // check error(s)
     if (0 == session)
     {
@@ -248,8 +257,7 @@ mfxStatus MFXClose(mfxSession session)
         // used after it. special care should be taken with MFX_AUTO_TRACE macro
         // since it inserts class variable on stack which calls to trace library in the
         // destructor.
-        MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXClose");
-        PERF_EVENT(MFX_TRACE_API_MFX_CLOSE_TASK, 0, make_event_data(session), [&](){ return make_event_data(mfxRes);});
+        MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
 
         // parent session can't be closed,
         // because there is no way to let children know about parent's death.
@@ -258,6 +266,7 @@ mfxStatus MFXClose(mfxSession session)
         if (session->IsChildSession())
         {
             mfxRes = MFXDisjoinSession(session);
+
             if (MFX_ERR_NONE != mfxRes)
             {
                 return mfxRes;
@@ -282,18 +291,19 @@ mfxStatus MFXClose(mfxSession session)
 #if defined(MFX_TRACE_ENABLE)
     MFX_TRACE_CLOSE();
 #endif
+    TRACE_EVENT(MFX_TRACE_API_MFX_CLOSE_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes));
+
     return mfxRes;
 
 } // mfxStatus MFXClose(mfxHDL session)
 
 mfxStatus MFX_CDECL MFXInitialize(mfxInitializationParam param, mfxSession* session)
 {
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
     mfxStatus mfxRes = MFX_ERR_NONE;
 
     MFX_TRACE_INIT();
-    PERF_EVENT(MFX_TRACE_API_MFXINITIALIZE_TASK, 0, make_event_data((mfxU32)param.AccelerationMode, param.VendorImplID),
-        [&](){ return make_event_data(mfxRes, session ? *session : nullptr); }
-    );
+    TRACE_EVENT(MFX_TRACE_API_MFXINITIALIZE_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data((mfxU32)param.AccelerationMode, param.VendorImplID));
 
     mfxInitParam par = {};
 
@@ -352,6 +362,8 @@ mfxStatus MFX_CDECL MFXInitialize(mfxInitializationParam param, mfxSession* sess
     // app. supposed just to copy VendorImplID from mfxImplDescription (returned by MFXQueryImplsDescription) to mfxInitializationParam
     mfxRes = MFXInit_Internal(par, session, par.Implementation, param.VendorImplID, isSingleThreadMode);
 
+    TRACE_EVENT(MFX_TRACE_API_MFXINITIALIZE_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(par.Implementation, isSingleThreadMode));
+
     return mfxRes;
 }
 
@@ -634,11 +646,13 @@ mfxHDL* MFX_CDECL MFXQueryImplsDescription(mfxImplCapsDeliveryFormat format, mfx
         return impl;
 
     MFX_TRACE_INIT();
+    MFXTrace_EventInit();
+    if (EnableEventTrace)
+    {
+        TRACE_EVENT(MFX_TRACE_API_MFXQUERYIMPLSDESCRIPTION_TASK, EVENT_TYPE_START, 0, make_event_data((mfxU32)format));
+    }
 
-    PERF_EVENT(MFX_TRACE_API_MFXQUERYIMPLSDESCRIPTION_TASK, 0, make_event_data((mfxU32)format),
-        [&]() { return make_event_data(*num_impls); }
-    );
-
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
     try
     {
         switch (format)
@@ -658,6 +672,9 @@ mfxHDL* MFX_CDECL MFXQueryImplsDescription(mfxImplCapsDeliveryFormat format, mfx
             holder->Remove("MFXVideoPAK_");
 
             *num_impls = mfxU32(holder->GetSize());
+
+            TRACE_EVENT(MFX_TRACE_API_MFXQUERYIMPLSDESCRIPTION_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(*num_impls));
+
             return holder.release()->GetArray();
         }
 #if defined(ONEVPL_EXPERIMENTAL)
@@ -701,6 +718,8 @@ mfxHDL* MFX_CDECL MFXQueryImplsDescription(mfxImplCapsDeliveryFormat format, mfx
 
             *num_impls = mfxU32(holder->GetSize());
 
+            TRACE_EVENT(MFX_TRACE_API_MFXQUERYIMPLSDESCRIPTION_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(*num_impls));
+
             holder->Detach();
             impl = holder.release()->GetArray();
 
@@ -769,6 +788,9 @@ mfxHDL* MFX_CDECL MFXQueryImplsDescription(mfxImplCapsDeliveryFormat format, mfx
                 auto logSkip = GetMfxLogSkip();
                 std::ignore  = logSkip;
 
+                InitMfxLogging();
+                MFX_LOG_API_TRACE("----------------MFXQueryImplsDescription----------------\n");
+
                 if (!QueryImplCaps(QueryImplDesc))
                     return impl;
             }
@@ -796,6 +818,7 @@ mfxHDL* MFX_CDECL MFXQueryImplsDescription(mfxImplCapsDeliveryFormat format, mfx
 
 mfxStatus MFX_CDECL MFXReleaseImplDescription(mfxHDL hdl)
 {
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
     MFX_CHECK_HDL(hdl);
 
     try
@@ -809,3 +832,4 @@ mfxStatus MFX_CDECL MFXReleaseImplDescription(mfxHDL hdl)
 
     return MFX_ERR_NONE;
 }
+
