@@ -62,6 +62,9 @@
 #include "mfx_av1_dec_decode.h"
 #endif
 
+
+#include "mfx_unified_decode_logging.h"
+
 template<>
 VideoDECODE* _mfxSession::Create<VideoDECODE>(mfxVideoParam& par)
 {
@@ -138,9 +141,13 @@ VideoDECODE* _mfxSession::Create<VideoDECODE>(mfxVideoParam& par)
 
 mfxStatus MFXVideoDECODE_Query(mfxSession session, mfxVideoParam *in, mfxVideoParam *out)
 {
+    InitMfxLogging();
+
     MFX_CHECK(session, MFX_ERR_INVALID_HANDLE);
     MFX_CHECK(out, MFX_ERR_NULL_PTR);
 
+    TRACE_EVENT(MFX_TRACE_API_DECODE_QUERY_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data(session, in ? in->mfx.FrameInfo.Width : 0, in ? in->mfx.FrameInfo.Height : 0, in ? in->mfx.CodecId : 0));
+
 #ifndef ANDROID
     if ((0 != in) && (MFX_HW_VAAPI == session->m_pCORE->GetVAType()))
     {
@@ -155,9 +162,8 @@ mfxStatus MFXVideoDECODE_Query(mfxSession session, mfxVideoParam *in, mfxVideoPa
 
     mfxStatus mfxRes;
 
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXVideoDECODE_Query");
-    PERF_EVENT(MFX_TRACE_API_DECODE_QUERY_TASK, 0, make_event_data(session, in ? in->mfx.FrameInfo.Width : 0, in ? in->mfx.FrameInfo.Height : 0, in ? in->mfx.CodecId : 0), [&](){ return make_event_data(mfxRes);});
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, in);
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, in);
 
     try
     {
@@ -214,6 +220,8 @@ mfxStatus MFXVideoDECODE_Query(mfxSession session, mfxVideoParam *in, mfxVideoPa
         default:
             mfxRes = MFX_ERR_UNSUPPORTED;
         }
+
+        TRACE_EVENT(MFX_TRACE_API_DECODE_QUERY_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes));
     }
     // handle error(s)
     catch(...)
@@ -221,22 +229,24 @@ mfxStatus MFXVideoDECODE_Query(mfxSession session, mfxVideoParam *in, mfxVideoPa
         mfxRes = MFX_ERR_UNKNOWN;
     }
 
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, out);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, out);
     MFX_LTRACE_I(MFX_TRACE_LEVEL_API, mfxRes);
     return mfxRes;
 }
 
 mfxStatus MFXVideoDECODE_QueryIOSurf(mfxSession session, mfxVideoParam *par, mfxFrameAllocRequest *request)
 {
+    InitMfxLogging();
+
     MFX_CHECK(session, MFX_ERR_INVALID_HANDLE);
     MFX_CHECK(par, MFX_ERR_NULL_PTR);
     MFX_CHECK(request, MFX_ERR_NULL_PTR);
+    TRACE_EVENT(MFX_TRACE_API_DECODE_QUERY_IOSURF_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data(session, (uint32_t)par->mfx.FrameInfo.Width, (uint32_t)par->mfx.FrameInfo.Height, par->mfx.CodecId));
 
     mfxStatus mfxRes;
 
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXVideoDECODE_QueryIOSurf");
-    PERF_EVENT(MFX_TRACE_API_DECODE_QUERY_IOSURF_TASK, 0, make_event_data(session, par->mfx.FrameInfo.Width, par->mfx.FrameInfo.Height, par->mfx.CodecId), [&](){ return make_event_data(mfxRes);});
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, par);
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, par);
 
     try
     {
@@ -293,6 +303,8 @@ mfxStatus MFXVideoDECODE_QueryIOSurf(mfxSession session, mfxVideoParam *par, mfx
         default:
             mfxRes = MFX_ERR_UNSUPPORTED;
         }
+
+        TRACE_EVENT(MFX_TRACE_API_DECODE_QUERY_IOSURF_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes));
     }
     // handle error(s)
     catch(...)
@@ -300,7 +312,10 @@ mfxStatus MFXVideoDECODE_QueryIOSurf(mfxSession session, mfxVideoParam *par, mfx
         mfxRes = MFX_ERR_UNKNOWN;
     }
 
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, request);
+    TRACE_BUFFER_EVENT(MFX_TRACE_API_DECODE_QUERY_IOSURF_TASK, EVENT_TYPE_INFO, TR_KEY_MFX_API,
+        request, DecodeQueryParam, DECODE_QUERY);
+
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, request);
     MFX_LTRACE_I(MFX_TRACE_LEVEL_API, mfxRes);
     return mfxRes;
 }
@@ -310,13 +325,13 @@ mfxStatus MFXVideoDECODE_DecodeHeader(mfxSession session, mfxBitstream *bs, mfxV
     MFX_CHECK(session, MFX_ERR_INVALID_HANDLE);
     MFX_CHECK(bs, MFX_ERR_NULL_PTR);
     MFX_CHECK(par, MFX_ERR_NULL_PTR);
+    TRACE_EVENT(MFX_TRACE_API_DECODE_HEADER_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data(session, bs, bs ? bs->DataLength : 0));
 
     mfxStatus mfxRes;
 
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXVideoDECODE_DecodeHeader");
-    PERF_EVENT(MFX_TRACE_API_DECODE_HEADER_TASK, 0, make_event_data(session, bs, bs ? bs->DataLength : 0), [&](){ return make_event_data(mfxRes);});
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, bs);
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, par);
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, bs);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, par);
 
     try
     {
@@ -373,6 +388,8 @@ mfxStatus MFXVideoDECODE_DecodeHeader(mfxSession session, mfxBitstream *bs, mfxV
         default:
             mfxRes = MFX_ERR_UNSUPPORTED;
         }
+
+        TRACE_EVENT(MFX_TRACE_API_DECODE_HEADER_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes));
     }
     // handle error(s)
     catch(...)
@@ -386,15 +403,16 @@ mfxStatus MFXVideoDECODE_DecodeHeader(mfxSession session, mfxBitstream *bs, mfxV
 
 mfxStatus MFXVideoDECODE_Init(mfxSession session, mfxVideoParam *par)
 {
+    InitMfxLogging();
+    TRACE_EVENT(MFX_TRACE_API_DECODE_INIT_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data(session, par->mfx.FrameInfo.Width, par->mfx.FrameInfo.Height, par->mfx.CodecId));
+
     mfxStatus mfxRes;
 
     MFX_CHECK(session, MFX_ERR_INVALID_HANDLE);
     MFX_CHECK(par, MFX_ERR_NULL_PTR);
 
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXVideoDECODE_Init");
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, par);
-
-    PERF_EVENT(MFX_TRACE_API_DECODE_INIT_TASK, 0, make_event_data(session, par->mfx.FrameInfo.Width, par->mfx.FrameInfo.Height, par->mfx.CodecId), [&](){ return make_event_data(mfxRes);});
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, par);
 
     try
     {
@@ -407,6 +425,8 @@ mfxStatus MFXVideoDECODE_Init(mfxSession session, mfxVideoParam *par)
         }
 
         mfxRes = session->m_pDECODE->Init(par);
+
+        TRACE_EVENT(MFX_TRACE_API_DECODE_INIT_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes));
     }
     catch(...)
     {
@@ -414,6 +434,9 @@ mfxStatus MFXVideoDECODE_Init(mfxSession session, mfxVideoParam *par)
         mfxRes = MFX_ERR_UNKNOWN;
     }
 
+    TRACE_BUFFER_EVENT(MFX_TRACE_API_DECODE_INIT_TASK, EVENT_TYPE_INFO, TR_KEY_MFX_API,
+        par, DecodeInitParam, DECODE_INIT);
+
     MFX_LTRACE_I(MFX_TRACE_LEVEL_API, mfxRes);
     return mfxRes;
 }
@@ -421,12 +444,12 @@ mfxStatus MFXVideoDECODE_Init(mfxSession session, mfxVideoParam *par)
 mfxStatus MFXVideoDECODE_Close(mfxSession session)
 {
     mfxStatus mfxRes = MFX_ERR_NONE;
+    TRACE_EVENT(MFX_TRACE_API_DECODE_CLOSE_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data(session));
 
     MFX_CHECK(session, MFX_ERR_INVALID_HANDLE);
     MFX_CHECK(session->m_pScheduler, MFX_ERR_NOT_INITIALIZED);
 
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXVideoDECODE_Close");
-    PERF_EVENT(MFX_TRACE_API_DECODE_CLOSE_TASK, 0, make_event_data(session), [&](){ return make_event_data(mfxRes);});
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
 
     try
     {
@@ -441,6 +464,8 @@ mfxStatus MFXVideoDECODE_Close(mfxSession session)
         mfxRes = session->m_pDECODE->Close();
 
         session->m_pDECODE.reset(nullptr);
+
+        TRACE_EVENT(MFX_TRACE_API_DECODE_CLOSE_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes));
     }
     // handle error(s)
     catch(...)
@@ -456,11 +481,11 @@ mfxStatus MFXVideoDECODE_Close(mfxSession session)
 mfxStatus MFXVideoDECODE_DecodeFrameAsync(mfxSession session, mfxBitstream *bs, mfxFrameSurface1 *surface_work, mfxFrameSurface1 **surface_out, mfxSyncPoint *syncp)
 {
     mfxStatus mfxRes;
+    TRACE_EVENT(MFX_TRACE_API_DECODE_FRAME_ASYNC_TASK, EVENT_TYPE_START, TR_KEY_MFX_API, make_event_data(session, surface_work, bs ? bs->DataLength : 0));
 
-    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, "MFXVideoDECODE_DecodeFrameAsync");
-    PERF_EVENT(MFX_TRACE_API_DECODE_FRAME_ASYNC_TASK, 0, make_event_data(session, surface_work, bs ? bs->DataLength : 0), [&](){ return make_event_data(mfxRes, syncp ? *syncp : nullptr);});
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, bs);
-    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, surface_work);
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, bs);
+    MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, surface_work);
 
     MFX_CHECK(session, MFX_ERR_INVALID_HANDLE);
     MFX_CHECK(session->m_pScheduler, MFX_ERR_NOT_INITIALIZED);
@@ -529,6 +554,8 @@ mfxStatus MFXVideoDECODE_DecodeFrameAsync(mfxSession session, mfxBitstream *bs,
         {
             *syncp = syncPoint;
         }
+
+        TRACE_EVENT(MFX_TRACE_API_DECODE_FRAME_ASYNC_TASK, EVENT_TYPE_END, TR_KEY_MFX_API, make_event_data(mfxRes, *syncp));
     }
     // handle error(s)
     catch(...)
@@ -541,7 +568,7 @@ mfxStatus MFXVideoDECODE_DecodeFrameAsync(mfxSession session, mfxBitstream *bs,
     {
         if (surface_out && *surface_out)
         {
-            MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API, *surface_out);
+            MFX_LTRACE_BUFFER(MFX_TRACE_LEVEL_API_PARAMS, *surface_out);
         }
         if (syncp)
         {
