@@ -30,6 +30,8 @@
 #include <algorithm>
 
 
+#define MFX_RT_PLATFORM_METEORLAKE 51
+
 mfxExtBuffer* GetExtendedBuffer(mfxExtBuffer** extBuf, mfxU32 numExtBuf, mfxU32 id)
 {
     if (extBuf != 0)
@@ -79,6 +81,7 @@ mfxStatus CheckFrameInfoCommon(mfxFrameInfo  *info, mfxU32 /* codecId */)
     case MFX_FOURCC_P016:
     case MFX_FOURCC_Y216:
     case MFX_FOURCC_Y416:
+    case MFX_FOURCC_ABGR16F:
         break;
     default:
         MFX_RETURN(MFX_ERR_INVALID_VIDEO_PARAM);
@@ -285,16 +288,21 @@ mfxStatus CheckFrameInfoCodecs(mfxFrameInfo  *info, mfxU32 codecId)
         break;
     }
 
-    // HEVC HW supports both kind of shifts
-    if ((codecId != MFX_CODEC_HEVC) && (
-           info->FourCC == MFX_FOURCC_P010
-        || info->FourCC == MFX_FOURCC_P210
-        || info->FourCC == MFX_FOURCC_Y210
-        || info->FourCC == MFX_FOURCC_P016
-        || info->FourCC == MFX_FOURCC_Y216
-        || info->FourCC == MFX_FOURCC_Y416))
+    switch (codecId) 
     {
-        MFX_CHECK(info->Shift == 1, MFX_ERR_INVALID_VIDEO_PARAM);
+    case MFX_CODEC_HEVC:
+        break;
+    default:
+        if (info->FourCC == MFX_FOURCC_P010
+            || info->FourCC == MFX_FOURCC_P210
+            || info->FourCC == MFX_FOURCC_Y210
+            || info->FourCC == MFX_FOURCC_P016
+            || info->FourCC == MFX_FOURCC_Y216
+            || info->FourCC == MFX_FOURCC_Y416) 
+        {
+            MFX_CHECK(info->Shift == 1, MFX_ERR_INVALID_VIDEO_PARAM);
+        }
+        break;
     }
 
     return MFX_ERR_NONE;
@@ -419,10 +427,6 @@ mfxStatus CheckVideoParamDecoders(mfxVideoParam *in, eMFXHWType type)
     MFX_CHECK(!(in->IOPattern & MFX_IOPATTERN_OUT_VIDEO_MEMORY) || !(in->IOPattern & MFX_IOPATTERN_OUT_SYSTEM_MEMORY)
         , MFX_ERR_INVALID_VIDEO_PARAM);
 
-    MFX_CHECK(!in->mfx.DecodedOrder || in->mfx.CodecId == MFX_CODEC_JPEG
-                                    || in->mfx.CodecId == MFX_CODEC_AVC
-                                    || in->mfx.CodecId == MFX_CODEC_HEVC, MFX_ERR_UNSUPPORTED);
-
 
     sts = CheckDecodersExtendedBuffers(in);
     MFX_CHECK(sts >= MFX_ERR_NONE, sts);
@@ -496,6 +500,7 @@ mfxStatus CheckFramePointers(mfxFrameInfo const& info, mfxFrameData const& data)
 
         case MFX_FOURCC_YV12:
         case MFX_FOURCC_YUY2:
+        case MFX_FOURCC_I420:
         default:                     MFX_CHECK(data.Y && data.U && data.V, MFX_ERR_UNDEFINED_BEHAVIOR); break;
     }
 
@@ -560,7 +565,7 @@ mfxStatus CheckDecodersExtendedBuffers(mfxVideoParam const* par)
     static const mfxU32 g_decoderSupportedExtBuffersMJPEG[] = {MFX_EXTBUFF_JPEG_HUFFMAN,
                                                                MFX_EXTBUFF_DEC_VIDEO_PROCESSING,
                                                                MFX_EXTBUFF_JPEG_QT};
-  
+
 
     static const mfxU32 g_decoderSupportedExtBuffersAV1[] = {
 #ifndef MFX_DEC_VIDEO_POSTPROCESS_DISABLE
@@ -830,6 +835,10 @@ bool mfxVideoParamWrapper::CreateExtendedBuffer(mfxU32 bufferId)
         m_buffers.AddTypedBuffer<mfxExtCodingOption>(bufferId);
         break;
 
+    case MFX_EXTBUFF_CHROMA_LOC_INFO:
+        m_buffers.AddTypedBuffer<mfxExtChromaLocInfo>(bufferId);
+        break;
+
     default:
         assert(false);
         return false;
@@ -946,6 +955,7 @@ mfxU32 GetMinPitch(mfxU32 fourcc, mfxU16 width)
         case MFX_FOURCC_P8_TEXTURE:
         case MFX_FOURCC_NV12:
         case MFX_FOURCC_YV12:
+        case MFX_FOURCC_I420:
 #ifdef MFX_ENABLE_RGBP
         case MFX_FOURCC_RGBP:
 #endif
@@ -965,7 +975,8 @@ mfxU32 GetMinPitch(mfxU32 fourcc, mfxU16 width)
         case MFX_FOURCC_A2RGB10:     return width * 4;
 
         case MFX_FOURCC_ARGB16:
-        case MFX_FOURCC_ABGR16:      return width * 8;
+        case MFX_FOURCC_ABGR16:  
+        case MFX_FOURCC_ABGR16F:     return width * 8;
 
         case MFX_FOURCC_YUY2:
         case MFX_FOURCC_UYVY:        return width * 2;
@@ -1012,6 +1023,8 @@ mfxU8* GetFramePointer(mfxU32 fourcc, mfxFrameData const& data)
 
         case MFX_FOURCC_Y416:        return reinterpret_cast<mfxU8*>(data.U16); break;
 
+        case MFX_FOURCC_ABGR16F:     return reinterpret_cast<mfxU8*>(data.ABGRFP16); break;
+
         default:                     return data.Y;
     }
 }
@@ -1089,6 +1102,9 @@ mfxU16 BitDepthFromFourcc(mfxU32 fourcc)
     case MFX_FOURCC_Y416:
         return 12;
 
+    case MFX_FOURCC_ABGR16F:
+        return 16;
+
         // RGB formats
 #if defined (MFX_ENABLE_FOURCC_RGB565)
     case MFX_FOURCC_RGB565:
@@ -1200,6 +1216,8 @@ mfxPlatform MakePlatform(eMFXHWType type, mfxU16 device_id)
     case MFX_HW_DG2    :
                          platform.MediaAdapterType = MFX_MEDIA_DISCRETE;
                          platform.CodeName = MFX_PLATFORM_DG2;           break;
+    // MFX_PLATFORM_METEORLAKE is not ready in spec, use MFX_RT_PLATFORM_METEORLAKE until spec updated
+    case MFX_HW_MTL    : platform.CodeName = MFX_RT_PLATFORM_METEORLAKE; break;
     default:
                          platform.MediaAdapterType = MFX_MEDIA_UNKNOWN;
                          platform.CodeName = MFX_PLATFORM_UNKNOWN;       break;
