@@ -500,10 +500,10 @@ mfxStatus VideoVPPBase::QueryCaps(VideoCORE * core, MfxHwVideoProcessing::mfxVpp
     if(MFX_PLATFORM_HARDWARE == core->GetPlatformType() )
     {
         sts = VideoVPPHW::QueryCaps(core, caps);
-        caps.uFrameRateConversion= 1; // "1" means general FRC is supported. "Interpolation" modes descibed by caps.frcCaps
-        caps.uDeinterlacing      = 1; // "1" means general deinterlacing is supported
-        caps.uVideoSignalInfoInOut      = 1; // "1" means general VSII is supported
-        caps.uVideoSignalInfo    = 1; // "1" means general VSIN is supported
+        caps.uFrameRateConversion  = 1; // "1" means general FRC is supported. "Interpolation" modes descibed by caps.frcCaps
+        caps.uDeinterlacing        = 1; // "1" means general deinterlacing is supported
+        caps.uVideoSignalInfoInOut = 1; // "1" means general VSII is supported
+        caps.uVideoSignalInfo      = 1; // "1" means general VSIN is supported
 
         if (sts >= MFX_ERR_NONE)
            return sts;
@@ -645,7 +645,7 @@ mfxStatus VideoVPPBase::Query(VideoCORE * core, mfxVideoParam *in, mfxVideoParam
                     (in->ExtParam[i] != 0 && out->ExtParam[i] == 0))
                 {
                     //mfxSts = MFX_ERR_UNDEFINED_BEHAVIOR;
-                    mfxSts = MFX_ERR_NULL_PTR;
+                    mfxSts = MFX_ERR_UNSUPPORTED;
                     continue; // stop working with ExtParam[i]
                 }
 
@@ -842,6 +842,18 @@ mfxStatus VideoVPPBase::Query(VideoCORE * core, mfxVideoParam *in, mfxVideoParam
                         continue;
                     }
 #endif
+                    else if (MFX_EXTBUFF_VIDEO_SIGNAL_INFO_IN == in->ExtParam[i]->BufferId)
+                    {
+                        continue;
+                    }
+                    else if (MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO == in->ExtParam[i]->BufferId)
+                    {
+                        continue;
+                    }
+                    else if (MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN == in->ExtParam[i]->BufferId)
+                    {
+                        continue;
+                    }
                     else
                     {
                         out->ExtParam[i]->BufferId = 0;
@@ -895,6 +907,7 @@ mfxStatus VideoVPPBase::Query(VideoCORE * core, mfxVideoParam *in, mfxVideoParam
 #endif // MFX_ENABLE_FOURCC_RGB565
             out->vpp.In.FourCC != MFX_FOURCC_RGB4 &&
             out->vpp.In.FourCC != MFX_FOURCC_BGR4 &&
+            out->vpp.In.FourCC != MFX_FOURCC_ABGR16F &&
             out->vpp.In.FourCC != MFX_FOURCC_P010 &&
             out->vpp.In.FourCC != MFX_FOURCC_UYVY &&
             out->vpp.In.FourCC != MFX_FOURCC_P210 &&
@@ -1196,6 +1209,11 @@ mfxStatus VideoVPP_HW::InternalInit(mfxVideoParam *par)
                             || IsFilterFound(&m_pipelineList[0], (mfxU32)m_pipelineList.size(), MFX_EXTBUFF_VPP_FIELD_WEAVING)
                             || IsFilterFound(&m_pipelineList[0], (mfxU32)m_pipelineList.size(), MFX_EXTBUFF_VPP_FIELD_SPLITTING);
 
+    /* We call driver instead of kernel on XeHP_SDV+ for field weaving and splitting */
+    if ((IsFilterFound(&m_pipelineList[0], (mfxU32)m_pipelineList.size(), MFX_EXTBUFF_VPP_FIELD_WEAVING)
+      || IsFilterFound(&m_pipelineList[0], (mfxU32)m_pipelineList.size(), MFX_EXTBUFF_VPP_FIELD_SPLITTING))
+      && !VppCaps::IsSwFieldProcessingSupported(m_core->GetHWType()))
+        isSWFieldProcessing = false;
 #endif
 
     pCommonCore = QueryCoreInterface<CommonCORE>(m_core, MFXIVideoCORE_GUID);
