@@ -107,7 +107,10 @@ const mfxU32 g_TABLE_CONFIG [] =
     MFX_EXTBUFF_VPP_3DLUT,
     MFX_EXTBUFF_VPP_DENOISE2,
     MFX_EXTBUFF_VIDEO_SIGNAL_INFO_IN,
-    MFX_EXTBUFF_VIDEO_SIGNAL_INFO_OUT
+    MFX_EXTBUFF_VIDEO_SIGNAL_INFO_OUT,
+    MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO,
+    MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN,
+    MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT
 };
 
 
@@ -140,7 +143,13 @@ const mfxU32 g_TABLE_EXT_PARAM [] =
     MFX_EXTBUFF_VPP_3DLUT,
     MFX_EXTBUFF_VPP_DENOISE2,
     MFX_EXTBUFF_VIDEO_SIGNAL_INFO_IN,
-    MFX_EXTBUFF_VIDEO_SIGNAL_INFO_OUT
+    MFX_EXTBUFF_VIDEO_SIGNAL_INFO_OUT,
+    MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO,
+    MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN,
+    MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT,
+    MFX_EXTBUF_CAM_PIPECONTROL,
+    MFX_EXTBUF_CAM_3DLUT,
+    MFX_EXTBUF_CAM_FORWARD_GAMMA_CORRECTION
 };
 
 PicStructMode GetPicStructMode(mfxU16 inPicStruct, mfxU16 outPicStruct)
@@ -832,6 +841,21 @@ void ReorderPipelineListForQuality( std::vector<mfxU32> & pipelineList )
         newList[index] = MFX_EXTBUFF_VIDEO_SIGNAL_INFO_OUT;
         index++;
     }
+    if (IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO))
+    {
+        newList[index] = MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO;
+        index++;
+    }
+    if (IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN))
+    {
+        newList[index] = MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN;
+        index++;
+    }
+    if (IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT))
+    {
+        newList[index] = MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT;
+        index++;
+    }
     if( IsFilterFound( &pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_VPP_SCENE_ANALYSIS ) )
     {
         newList[index] = MFX_EXTBUFF_VPP_SCENE_ANALYSIS;
@@ -1070,7 +1094,7 @@ mfxStatus GetPipelineList(
     /* [Deinterlace] FILTER */
     if( 0 != videoParam->NumExtParam && NULL == videoParam->ExtParam )
     {
-        return MFX_ERR_NULL_PTR;
+        return MFX_ERR_INVALID_VIDEO_PARAM;
     }
     PicStructMode picStructMode = GetPicStructMode(par->In.PicStruct, par->Out.PicStruct);
 
@@ -1295,6 +1319,27 @@ mfxStatus GetPipelineList(
             pipelineList.push_back( MFX_EXTBUFF_VIDEO_SIGNAL_INFO_OUT );
         }
     }
+    if (IsFilterFound(&configList[0], configCount, MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN) && !IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN))
+    {
+        if (!IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN))
+        {
+            pipelineList.push_back(MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_IN);
+        }
+    }
+    if (IsFilterFound(&configList[0], configCount, MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT) && !IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT))
+    {
+        if (!IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT))
+        {
+            pipelineList.push_back(MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME_OUT);
+        }
+    }
+    if (IsFilterFound(&configList[0], configCount, MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO) && !IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO))
+    {
+        if (!IsFilterFound(&pipelineList[0], (mfxU32)pipelineList.size(), MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO))
+        {
+            pipelineList.push_back(MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO);
+        }
+    }
 
     searchCount = sizeof(g_TABLE_CONFIG) / sizeof(*g_TABLE_CONFIG);
     fCount      = configCount;
@@ -1398,6 +1443,14 @@ mfxStatus CheckFrameInfo(mfxFrameInfo* info, mfxU32 request)
         case MFX_FOURCC_Y216:
         case MFX_FOURCC_Y416:
             break;
+        case MFX_FOURCC_ABGR16F:
+        case MFX_FOURCC_I420:
+        case MFX_FOURCC_UYVY:
+            break;
+        case MFX_FOURCC_ARGB16:
+            break;
+        case MFX_FOURCC_R16:
+            break;
         case MFX_FOURCC_IMC3:
         case MFX_FOURCC_YUV400:
         case MFX_FOURCC_YUV411:
@@ -1405,7 +1458,6 @@ mfxStatus CheckFrameInfo(mfxFrameInfo* info, mfxU32 request)
         case MFX_FOURCC_YUV422V:
         case MFX_FOURCC_YUV444:
         // UYVY is supported as input on Linux only
-        case MFX_FOURCC_UYVY:
             if (VPP_OUT == request)
                 return MFX_ERR_INVALID_VIDEO_PARAM;
             break;
@@ -2263,6 +2315,16 @@ void ConvertCaps2ListDoUse(MfxHwVideoProcessing::mfxVppCaps& caps, std::vector<m
         list.push_back(MFX_EXTBUFF_VPP_FIELD_PROCESSING);
     }
 
+    if (caps.cameraCaps.uGammaCorrection)
+    {
+        list.push_back(MFX_EXTBUF_CAM_FORWARD_GAMMA_CORRECTION);
+    }
+
+    if (caps.cameraCaps.u3DLUT)
+    {
+        list.push_back(MFX_EXTBUF_CAM_3DLUT);
+    }
+
     /*ColorFill is always present*/
     list.push_back(MFX_EXTBUFF_VPP_COLORFILL);
     /* Field weaving is always present*/
