@@ -37,6 +37,13 @@
 #include <condition_variable>
 #include <algorithm>
 
+#ifdef MFX_ENABLE_ENCODE_STATS
+#include "mfxencodestats.h"
+#endif // MFX_ENABLE_ENCODE_STATS
+
+static const size_t BASE_ADDR_ALIGN = 0x1000; // 4k page size alignment
+static const size_t BASE_SIZE_ALIGN = 0x1000; // 4k page size alignment
+
 // Internal Allocators
 namespace mfxDefaultAllocator
 {
@@ -51,6 +58,8 @@ namespace mfxDefaultAllocator
     mfxStatus UnlockFrame(mfxHDL pthis, mfxMemId mid, mfxFrameData *ptr=0);
     mfxStatus FreeFrames(mfxHDL pthis, mfxFrameAllocResponse *response);
 
+    mfxStatus GetNumBytesRequired(const mfxFrameInfo& Info, mfxU32& nbytes, size_t power_of_2_alignment = BASE_SIZE_ALIGN);
+
     struct BufferStruct
     {
         mfxHDL      allocator;
@@ -806,12 +815,12 @@ struct mfxRefCountableInstance<mfxFrameSurface1>
 
 inline void copy_frame_surface_pixel_pointers(mfxFrameData& buf_dst, const mfxFrameData& buf_src)
 {
-    MFX_COPY_FIELD(PitchLow);
-    MFX_COPY_FIELD(PitchHigh);
-    MFX_COPY_FIELD(Y);
-    MFX_COPY_FIELD(U);
-    MFX_COPY_FIELD(V);
-    MFX_COPY_FIELD(A);
+    MFX_COPY_FIELD_NO_LOG(PitchLow);
+    MFX_COPY_FIELD_NO_LOG(PitchHigh);
+    MFX_COPY_FIELD_NO_LOG(Y);
+    MFX_COPY_FIELD_NO_LOG(U);
+    MFX_COPY_FIELD_NO_LOG(V);
+    MFX_COPY_FIELD_NO_LOG(A);
 }
 
 class mfxFrameSurfaceInterfaceImpl : public mfxFrameSurfaceBaseInterface
@@ -1016,6 +1025,120 @@ private:
     std::mutex m_mutex;
 };
 
+#ifdef MFX_ENABLE_ENCODE_STATS
+class mfxEncodeStatsContainerImpl : public mfxRefCountableImpl<mfxEncodeStatsContainer>
+{
+public:
+    mfxStatus AllocFrameStatsBuf()
+    {
+        std::lock_guard<std::mutex> guard(m_mutex);
+
+        if (this->EncodeFrameStats == nullptr)
+        {
+            this->EncodeFrameStats = new mfxEncodeFrameStats{};
+
+            MFX_CHECK(this->EncodeFrameStats, MFX_ERR_MEMORY_ALLOC);
+        }
+        else
+        {
+            *this->EncodeFrameStats = {};
+        }
+
+        return MFX_ERR_NONE;
+    }
+
+    mfxStatus AllocBlkStatsBuf(mfxU32 numBlk)
+    {
+        MFX_CHECK(numBlk, MFX_ERR_INVALID_VIDEO_PARAM);
+
+        std::lock_guard<std::mutex> guard(m_mutex);
+
+        if (this->EncodeBlkStats == nullptr)
+        {
+            this->EncodeBlkStats = new mfxEncodeBlkStats{};
+            MFX_CHECK(this->EncodeBlkStats, MFX_ERR_MEMORY_ALLOC);
+        }
+
+        MFX_CHECK_STS(AllocBlkStatsArray(numBlk));
+
+        return MFX_ERR_NONE;
+    }
+
+protected:
+    template <typename T>
+    static void Delete(T*& p)
+    {
+        if (p)
+        {
+            delete p;
+            p = nullptr;
+        }
+    }
+
+    template <typename T>
+    static void DeleteArray(T*& p)
+    {
+        if (p)
+        {
+            delete[] p;
+            p = nullptr;
+        }
+    }
+
+    template <typename T>
+    static mfxStatus AllocBlkStatsBuf(mfxU32 numBlkIn, mfxU32& numBlkOut, T*& buf)
+    {
+        if (numBlkIn <= numBlkOut && buf)
+        {
+            for (mfxU32 i = 0; i < numBlkOut; i++)
+            {
+                buf[i] = {};
+            }
+
+            return MFX_ERR_NONE;
+        }
+
+        if (numBlkOut)
+        {
+            DeleteArray(buf);
+        }
+
+        buf = new T[numBlkIn]{};
+        MFX_CHECK(buf, MFX_ERR_MEMORY_ALLOC);
+        numBlkOut = numBlkIn;
+
+        return MFX_ERR_NONE;
+    }
+
+    virtual mfxStatus AllocBlkStatsArray(mfxU32 numBlk) = 0;
+
+    virtual void DetroyBlkStatsArray() = 0;
+
+    void Close() override
+    {
+        if (this->EncodeFrameStats)
+        {
+            Delete(this->EncodeFrameStats);
+        }
+
+        if (this->EncodeBlkStats)
+        {
+            DetroyBlkStatsArray();
+            Delete(this->EncodeBlkStats);
+        }
+    }
+
+protected:
+    mfxEncodeStatsContainerImpl()
+    {
+        Version.Version      = MFX_ENCODESTATSCONTAINER_VERSION;
+        RefInterface.Context = static_cast<mfxRefCountable*>(this);
+    }
+
+    std::mutex m_mutex;
+};
+#endif // MFX_ENABLE_ENCODE_STATS
+
 class RWAcessSurface : public mfxFrameSurfaceInterfaceImpl
 {
 public:
