@@ -32,6 +32,8 @@
 #include <stdarg.h>
 
 #include "mfx_config.h"
+#include "mfx_trace_dump.h"
+#include "mfx_error.h"
 
     #define MAX_PATH 260
 
@@ -52,6 +54,11 @@ typedef unsigned int mfxTraceU32;
 typedef __UINT64 mfxTraceU64;
 
 /*------------------------------------------------------------------------------*/
+extern mfxTraceU32 EnableEventTrace;
+extern mfxTraceU32 EventCfg;
+extern mfxTraceU32 LogConfig;
+extern int32_t FrameIndex;
+
 // C section
 
 #ifdef __cplusplus
@@ -59,7 +66,7 @@ extern "C"
 {
 #endif
 
-// this is for perf events
+// this is for RT events
 enum mfxTraceTaskType
 {
     MFX_TRACE_DEFAULT_TASK = 0,
@@ -87,6 +94,8 @@ enum mfxTraceTaskType
     MFX_TRACE_API_SYNC_OPERATION_TASK,
     MFX_TRACE_HOTSPOT_SCHED_WAIT_GLOBAL_EVENT_TASK,
     MFX_TRACE_HOTSPOT_DDI_SUBMIT_TASK,
+    MFX_TRACE_HOTSPOT_DDI_EXECUTE_TASK,
+    MFX_TRACE_HOTSPOT_DDI_ENDFRAME_TASK,
     MFX_TRACE_HOTSPOT_DDI_WAIT_TASK_SYNC,
     MFX_TRACE_HOTSPOT_DDI_QUERY_TASK,
     MFX_TRACE_HOTSPOT_CM_COPY,
@@ -95,6 +104,46 @@ enum mfxTraceTaskType
     MFX_TRACE_HOTSPOT_SCHED_ROUTINE,
     MFX_TRACE_API_MFXINITIALIZE_TASK,
     MFX_TRACE_API_MFXQUERYIMPLSDESCRIPTION_TASK,
+    MFX_TRACE_API_HEVC_OUTPUTINFO_TASK,
+    MFX_TRACE_API_AVC_OUTPUTINFO_TASK,
+    MFX_TRACE_API_AV1_OUTPUTINFO_TASK,
+    MFX_TRACE_API_VP9_OUTPUTINFO_TASK,
+    MFX_TRACE_API_HEVC_SYNCINFO_TASK,
+    MFX_TRACE_API_AVC_SYNCINFO_TASK,
+    MFX_TRACE_API_AV1_SYNCINFO_TASK,
+    MFX_TRACE_API_VP9_SYNCINFO_TASK,
+    MFX_TRACE_API_HEVC_DISPLAYINFO_TASK,
+    MFX_TRACE_API_AVC_DISPLAYINFO_TASK,
+    MFX_TRACE_API_AV1_DISPLAYINFO_TASK,
+    MFX_TRACE_API_VP9_DISPLAYINFO_TASK,
+    MFX_TRACE_API_HEVC_PICTUREPARAMETER_TASK,   
+    MFX_TRACE_API_HEVC_SLICEPARAMETER_TASK,
+    MFX_TRACE_API_HEVC_QMATRIXARAMETER_TASK,
+    MFX_TRACE_API_AVC_PICTUREPARAMETER_TASK,
+    MFX_TRACE_API_AVC_SLICEPARAMETER_TASK,
+    MFX_TRACE_API_AVC_QMATRIXARAMETER_TASK,
+    MFX_TRACE_API_AV1_PICTUREPARAMETER_TASK,
+    MFX_TRACE_API_AV1_TILECONTROLPARAMETER_TASK,      
+    MFX_TRACE_API_VP9_PICTUREPARAMETER_TASK,
+    MFX_TRACE_API_VP9_SEGMENTPARAMETER_TASK,   
+    MFX_TRACE_API_BITSTREAM_TASK,
+    MFX_TRACE_API_HEVC_DPBPARAMETER_TASK,
+    MFX_TRACE_API_AVC_DPBPARAMETER_TASK,
+    MFX_TRACE_API_AV1_DPBPARAMETER_TASK,
+    MFX_TRACE_API_VP9_DPBPARAMETER_TASK,
+    VA_TRACE_API_AV1_PICTUREPARAMETER_TASK,
+    VA_TRACE_API_AV1_TILECONTROLPARAMETER_TASK,
+    VA_TRACE_API_AVC_PICTUREPARAMETER_TASK,
+    VA_TRACE_API_AVC_SLICEPARAMETER_TASK,
+    VA_TRACE_API_AVC_QMATRIXARAMETER_TASK,
+    VA_TRACE_API_HEVC_PICTUREPARAMETER_TASK,
+    VA_TRACE_API_HEVC_SLICEPARAMETER_TASK,
+    VA_TRACE_API_HEVC_QMATRIXARAMETER_TASK,
+    VA_TRACE_API_VP9_PICTUREPARAMETER_TASK,
+    VA_TRACE_API_VP9_SLICEPARAMETER_TASK,
+    VA_TRACE_API_HEVC_DPBPARAMETER_TASK,
+    MFX_TRACE_PIPELINE_STICKER_TASK,
+    VPLMessage_TASK,
 };
 
 // list of output modes
@@ -137,6 +186,53 @@ typedef enum
     MFX_TRACE_LEVEL_MAX = 0xFF
 } mfxTraceLevel;
 
+// enumeration of the TXT log levels
+typedef enum
+{
+#ifndef NDEBUG
+    MFX_TXTLOG_LEVEL_MAX = 1,   //include API Func, API PARAMS and internal Func
+    MFX_TXTLOG_LEVEL_API_AND_INTERNAL = 2,  //include API Func and internal Func
+    MFX_TXTLOG_LEVEL_API_AND_PARAMS = 3, //include API Func and API PARAMS
+    MFX_TXTLOG_LEVEL_API = 4,   //include API Func
+#else
+    MFX_TXTLOG_LEVEL_API_AND_PARAMS = 1,    //include API Func, API PARAMS
+    MFX_TXTLOG_LEVEL_API = 2    //include API Func
+#endif
+} mfxTxtLogLevel;
+
+typedef enum _MEDIA_EVENT_TYPE
+{
+    EVENT_TYPE_INFO = 0,           //! function information event
+    EVENT_TYPE_START = 1,           //! function entry event
+    EVENT_TYPE_END = 2,           //! function exit event
+    EVENT_TYPE_INFO2 = 3,           //! function extra information event
+} MEDIA_EVENT_TYPE;
+
+//!
+//! \brief Keyword for ETW tracing, 1bit per keyworld, total 64bits
+//!
+typedef enum _MEDIA_EVENT_FILTER_KEYID
+{
+    //Common key
+    TR_KEY_MFX_API = 0,
+    TR_KEY_DDI_API,
+    //Decode key
+    TR_KEY_DECODE_PICPARAM,
+    TR_KEY_DECODE_SLICEPARAM,
+    TR_KEY_DECODE_TILEPARAM,
+    TR_KEY_DECODE_QMATRIX,
+    TR_KEY_DECODE_SEGMENT,
+    TR_KEY_DECODE_BITSTREAM_INFO,
+    TR_KEY_DECODE_DPB_INFO,
+    TR_KEY_DECODE_BASIC_INFO,
+    TR_KEY_PIPELINE_STICKER,
+} MEDIA_EVENT_FILTER_KEYID;
+
+typedef enum _MEDIA_EVENT_LOG_KEYID
+{
+    MFX_ETWLOG_ALL_LEVEL = 16,   //include API Func, API PARAMS and internal Func
+    MFX_ETWLOG_API_LEVEL,   //include API Func
+} MEDIA_EVENT_LOG_KEYID;
 
 // delete the following levels completely
 #define MFX_TRACE_LEVEL_SCHED       MFX_TRACE_LEVEL_10
@@ -163,7 +259,7 @@ typedef enum
 /** SCHEDULER level
  * - Media SDK internal scheduler functions calls
  */
-#define MFX_TRACE_LEVEL_SCHEDULER   MFX_TRACE_LEVEL_4
+#define MFX_TRACE_LEVEL_API_PARAMS   MFX_TRACE_LEVEL_4
 
 /** INTERNAL level
  * - Media SDK components function calls. Use this level to get more deeper knowledge of
@@ -193,8 +289,8 @@ typedef enum
 #endif
 
 // Perf traces
-mfxTraceU32 MFXTrace_PerfInit();
-mfxTraceU32 MFXTrace_PerfClose();
+mfxTraceU32 MFXTrace_EventInit();
+mfxTraceU32 MFXTrace_EventClose();
 
 /*------------------------------------------------------------------------------*/
 
@@ -285,7 +381,7 @@ mfxTraceU32 MFXTrace_EndTask(mfxTraceStaticHandle *static_handle,
 /*------------------------------------------------------------------------------*/
 
 // Perf traces
-mfxTraceU32 MFXTrace_PerfEvent(uint16_t task, uint8_t opcode, uint8_t level, uint64_t size, void *ptr);
+mfxTraceU32 MFXTraceEvent(uint16_t task, uint8_t opcode, uint8_t level, uint64_t size, const void *ptr);
 
 #ifdef __cplusplus
 }
@@ -342,35 +438,27 @@ auto make_event_data(Args&&... args)
 // PerfScopedTrace objects are created using PERF_EVENT macro
 class PerfScopedTrace
 {
-    std::function<void()> at_exit;
-
 public:
-    template <typename Func, typename ...Args>
-    PerfScopedTrace(uint16_t task, uint8_t level, event_data<Args...> data, Func at_exit_func)
+    template <typename ...Args>
+    PerfScopedTrace(uint16_t task, uint8_t ucType, uint8_t level, event_data<Args...> data)
     {
-        using F = typename std::decay<Func>::type;
-        auto f = new F(at_exit_func);
-        // Bind at_exit function to trace values at the end of the function execution
-        at_exit = std::bind(get_thunk<F>(), f, task, level);
-
         // Send event data on PerfScopedTrace creation
-        write(task, 1, level, data);
+        write(task, ucType, level, data);
     }
 
     ~PerfScopedTrace()
     {
-        at_exit();
     }
 
     template <typename T>
     static void write(uint16_t task, uint8_t  opcode, uint8_t level, T&& data)
     {
-        MFXTrace_PerfEvent(task, opcode, level, sizeof(data), &data);
+        MFXTraceEvent(task, opcode, level, sizeof(data), &data);
     }
 
     static void write(uint16_t task, uint8_t  opcode, uint8_t level, event_data<>& /*data*/)
     {
-        MFXTrace_PerfEvent(task, opcode, level, 0, nullptr);
+        MFXTraceEvent(task, opcode, level, 0, nullptr);
     }
 
     template <typename T>
@@ -390,10 +478,23 @@ public:
     }
 };
 
-// This macro is recommended to use instead creating PerfScopedTrace object directly
-#define PERF_EVENT(task, level, data, at_exit_func) \
-    PerfScopedTrace _perf_scoped_trace##__LINE__ (task, level, data, at_exit_func)
+#define TRACE_CHECK(keyWord)   \
+    (EventCfg & (1 << keyWord) || keyWord == TR_KEY_MFX_API || keyWord == TR_KEY_DDI_API) && EnableEventTrace
 
+// This macro is recommended to use instead creating RT Info ScopedTrace object directly
+#define TRACE_BUFFER_EVENT(task, level, keyWord, pData, funcName, structType)   \
+    if (TRACE_CHECK(keyWord))                           \
+    {                                                   \
+        EVENTDATA_##structType eventData;               \
+        Event##funcName(&eventData, pData);             \
+        PerfScopedTrace _info_scoped_trace##__LINE__(task, level, 0, make_event_data(eventData));   \
+    }
+
+#define TRACE_EVENT(task, level, keyWord, at_exit_func)         \
+    if (TRACE_CHECK(keyWord))                                   \
+    {                                                           \
+        PerfScopedTrace _info_scoped_trace##__LINE__(task, level, 0, at_exit_func);    \
+    }
 
 #ifdef __cplusplus
 extern "C" {
@@ -433,6 +534,7 @@ extern "C" {
 #define MFX_TRACE_CLOSE_RES(res)
 #define MFX_LTRACE(_trace_all_params)
 #define PERF_EVENT(task, level, ...)
+#define TRACE_EVENT(task, level, ...)
 #endif
 
 /*------------------------------------------------------------------------------*/
@@ -476,9 +578,6 @@ extern "C" {
 #define MFX_LTRACE_P(_level, _arg1) \
     MFX_LTRACE_1(_level, #_arg1 " = ", MFX_TRACE_FORMAT_P, _arg1)
 
-#define MFX_LTRACE_I(_level, _arg1) \
-    MFX_LTRACE_1(_level, #_arg1 " = ", MFX_TRACE_FORMAT_I, _arg1)
-
 #define MFX_LTRACE_X(_level, _arg1) \
     MFX_LTRACE_1(_level, #_arg1 " = ", MFX_TRACE_FORMAT_X, _arg1)
 
@@ -496,8 +595,23 @@ extern "C" {
 #define MFX_LTRACE_BUFFER_S(_level, _name, _buffer, _size)
 #endif
 
+#ifdef MFX_TRACE_ENABLE
 #define MFX_LTRACE_BUFFER(_level, _buffer) \
-    MFX_LTRACE_BUFFER_S(_level, #_buffer, _buffer, sizeof(*_buffer)) \
+{ \
+    if (0 != LogConfig && _buffer) \
+    { \
+        DumpContext context; \
+        std::string _str = context.dump(#_buffer, *_buffer); \
+        MFX_LTRACE_1(_level, "", "\n%s", _str.c_str()) \
+    } \
+    else { \
+        MFX_LTRACE_BUFFER_S(_level, #_buffer, _buffer, sizeof(*_buffer)) \
+    } \
+}
+#else
+#define MFX_LTRACE_BUFFER(_level, _buffer) \
+    MFX_LTRACE_BUFFER_S(_level, #_buffer, _buffer, sizeof(*_buffer)) 
+#endif
 
 #define MFX_LTRACE_GUID(_level, _guid) \
     MFX_LTRACE((MFX_TRACE_PARAMS, _level, #_guid " = ", \
@@ -619,6 +733,37 @@ private:
 #define MFX_AUTO_TRACE_WITHID(_task_name) \
     _MFX_AUTO_LTRACE_(MFX_TRACE_LEVEL, _task_name, MFX_TRACE_DEFAULT_TASK, true)
 
+#ifdef MFX_TRACE_ENABLE
+class MFXLTraceI 
+{
+public:
+    template <typename T>
+    void mfx_ltrace_i(mfxTraceLevel _level, const char* _mesg, const char* _function, T _arg1)
+    {
+        std::stringstream ss;
+        ss << _mesg << " = ";
+        MFX_LTRACE((&_trace_static_handle, __FILE__, __LINE__, _function, MFX_TRACE_CATEGORY, _level, ss.str().c_str(), MFX_TRACE_FORMAT_I, _arg1))
+    }
+
+    void mfx_ltrace_i(mfxTraceLevel _level, const char* _mesg, const char* _function, mfxStatus _arg1)
+    {
+        std::error_code code = mfx::make_error_code(_arg1);
+        std::stringstream ss;
+        ss << _mesg << " = ";
+        MFX_LTRACE((&_trace_static_handle, __FILE__, __LINE__, _function, MFX_TRACE_CATEGORY, _level, ss.str().c_str(), MFX_TRACE_FORMAT_S, code.message().c_str()))
+    }
+};
+
+#define MFX_LTRACE_I(_level, _arg1) \
+{ \
+    MFXLTraceI mFXLTraceI; \
+    mFXLTraceI.mfx_ltrace_i(_level, #_arg1, __FUNCTION__, _arg1); \
+}
+#else
+#define MFX_LTRACE_I(_level, _arg1) \
+    MFX_LTRACE_1(_level, #_arg1 " = ", MFX_TRACE_FORMAT_I, _arg1)
+#endif
+
 #endif // ifdef __cplusplus
 
 template <int TYPE>
@@ -675,6 +820,10 @@ struct TraceTaskType2TraceLevel<MFX_TRACE_HOTSPOT_SCHED_WAIT_GLOBAL_EVENT_TASK>
 template <>
 struct TraceTaskType2TraceLevel<MFX_TRACE_HOTSPOT_DDI_SUBMIT_TASK> : std::integral_constant<mfxTraceLevel, MFX_TRACE_LEVEL_HOTSPOTS> {};
 template <>
+struct TraceTaskType2TraceLevel<MFX_TRACE_HOTSPOT_DDI_EXECUTE_TASK> : std::integral_constant<mfxTraceLevel, MFX_TRACE_LEVEL_HOTSPOTS> {};
+template <>
+struct TraceTaskType2TraceLevel<MFX_TRACE_HOTSPOT_DDI_ENDFRAME_TASK> : std::integral_constant<mfxTraceLevel, MFX_TRACE_LEVEL_HOTSPOTS> {};
+template <>
 struct TraceTaskType2TraceLevel<MFX_TRACE_HOTSPOT_DDI_WAIT_TASK_SYNC> : std::integral_constant<mfxTraceLevel, MFX_TRACE_LEVEL_HOTSPOTS> {};
 template <>
 struct TraceTaskType2TraceLevel<MFX_TRACE_HOTSPOT_DDI_QUERY_TASK> : std::integral_constant<mfxTraceLevel, MFX_TRACE_LEVEL_HOTSPOTS> {};
