@@ -46,6 +46,8 @@
 #include <atomic>
 #include <sstream>
 #include <utility>
+#include <malloc.h>
+#include <cstdlib>
 
 #if defined(MFX_ENABLE_LOG_UTILITY)
 template <
@@ -165,8 +167,33 @@ bool LumaIsNull(const mfxFrameSurface1 * surf)
 
 #define IS_PROTECTION_ANY(val) (false)
 
+#define MFX_COPY_FIELD_NO_LOG(Field)       buf_dst.Field = buf_src.Field
+
+#if !defined(MFX_ENABLE_LOG_UTILITY)
 #define MFX_COPY_FIELD(Field)       buf_dst.Field = buf_src.Field
 #define MFX_COPY_ARRAY_FIELD(Array) std::copy(std::begin(buf_src.Array), std::end(buf_src.Array), std::begin(buf_dst.Array))
+#else
+#define MFX_COPY_FIELD(Field)                                                                   \
+    buf_dst.Field = buf_src.Field;                                                              \
+    {                                                                                           \
+        const std::string fieldForamt = GetNumberFormat(buf_src.Field);                         \
+        const std::string typeName    = std::string(typeid(buf_src).name());                    \
+        const std::string format      = typeName + ".%s = " + fieldForamt + "\n";               \
+        MFX_LOG_API_TRACE(format.c_str(), #Field, buf_src.Field);                               \
+    }
+
+#define MFX_COPY_ARRAY_FIELD(Array)                                                             \
+    std::copy(std::begin(buf_src.Array), std::end(buf_src.Array), std::begin(buf_dst.Array));   \
+    {                                                                                           \
+        int idx = 0;                                                                            \
+        const std::string fieldForamt = GetNumberFormat(buf_src.Array[0]);                      \
+        const std::string typeName    = std::string(typeid(buf_src).name());                    \
+        const std::string format      = typeName + ".%s[%d] = " + fieldForamt + "\n";           \
+        for (auto it = std::begin(buf_src.Array); it != std::end(buf_src.Array); it++, idx++) { \
+            MFX_LOG_API_TRACE(format.c_str(), #Array, idx, *it);                                \
+        }                                                                                       \
+    }
+#endif
 
 namespace mfx
 {
@@ -784,6 +811,28 @@ class mfxRefCountableImpl
     : public mfxRefCountable
     , public T
 {
+protected:
+    template <typename X, typename = void>
+    struct HasRefInterface : std::false_type { };
+
+    template <typename X>
+    struct HasRefInterface <X, decltype(X::RefInterface, void())> : std::true_type { };
+
+    template <typename X, typename std::enable_if<HasRefInterface<X>::value, bool>::type = true>
+    void AssignFunctionPointers()
+    {
+        T::RefInterface.AddRef        = _AddRef2;
+        T::RefInterface.Release       = _Release2;
+        T::RefInterface.GetRefCounter = _GetRefCounter2;
+    }
+
+    template <typename X, typename std::enable_if<!HasRefInterface<X>::value, bool>::type = true>
+    void AssignFunctionPointers()
+    {
+        T::AddRef        = _AddRef;
+        T::Release       = _Release;
+        T::GetRefCounter = _GetRefCounter;
+    }
 public:
     mfxRefCountableImpl()
         : T()
@@ -793,9 +842,7 @@ public:
         g_global_registry.RegisterRefcountObject(this);
 #endif
 
-        T::AddRef          = _AddRef;
-        T::Release         = _Release;
-        T::GetRefCounter   = _GetRefCounter;
+        AssignFunctionPointers<T>();
     }
 
     virtual ~mfxRefCountableImpl()
@@ -869,6 +916,37 @@ public:
         return MFX_ERR_NONE;
     }
 
+    static mfxStatus _AddRef2(mfxRefInterface* object)
+    {
+        MFX_CHECK_NULL_PTR1(object);
+        auto instance = static_cast<mfxRefCountable*>(object->Context);
+        MFX_CHECK_HDL(instance);
+
+        instance->AddRef();
+        return MFX_ERR_NONE;
+    }
+
+    static mfxStatus _Release2(mfxRefInterface* object)
+    {
+        MFX_CHECK_NULL_PTR1(object);
+        auto instance = static_cast<mfxRefCountable*>(object->Context);
+        MFX_CHECK_HDL(instance);
+
+        return instance->Release();
+    }
+
+    static mfxStatus _GetRefCounter2(mfxRefInterface* object, mfxU32* counter)
+    {
+        MFX_CHECK_NULL_PTR1(object);
+        MFX_CHECK_NULL_PTR1(counter);
+
+        auto instance = static_cast<mfxRefCountable*>(object->Context);
+        MFX_CHECK_HDL(instance);
+
+        *counter = instance->GetRefCounter();
+        return MFX_ERR_NONE;
+    }
+
 protected:
 
     virtual void Close() { return; }
@@ -975,62 +1053,6 @@ struct surface_cache_controller
 
     mfxStatus SetupCache(mfxSession session, const mfxVideoParam& par)
     {
-        // Use temporal copy for possible IOPattern adjustment. Hitorically Init doesn't require setting of IOPatter, but QueryIOSurf does.
-        // This function is invoked during Init
-        mfxVideoParam tmp_par = par;
-
-        switch (m_type)
-        {
-        case ComponentType::DECODE:
-        {
-            mfxFrameAllocRequest req = {};
-
-            if (!tmp_par.IOPattern)
-                tmp_par.IOPattern = MFX_IOPATTERN_OUT_VIDEO_MEMORY;
-
-            mfxStatus sts = MFX_STS_TRACE(MFXVideoDECODE_QueryIOSurf(session, &tmp_par, &req));
-            MFX_CHECK(sts >= MFX_ERR_NONE, sts);
-
-            m_required_num_surf = req.NumFrameSuggested;
-        }
-        break;
-
-        case ComponentType::ENCODE:
-        {
-            mfxFrameAllocRequest req = {};
-
-            if (!tmp_par.IOPattern)
-                tmp_par.IOPattern = MFX_IOPATTERN_IN_VIDEO_MEMORY;
-
-            mfxStatus sts = MFX_STS_TRACE(MFXVideoENCODE_QueryIOSurf(session, &tmp_par, &req));
-            MFX_CHECK(sts >= MFX_ERR_NONE, sts);
-
-            m_required_num_surf = req.NumFrameSuggested;
-        }
-        break;
-
-        case ComponentType::VPP:
-        {
-            mfxFrameAllocRequest req[2] = {};
-
-            if (!tmp_par.IOPattern)
-                tmp_par.IOPattern = MFX_IOPATTERN_IN_VIDEO_MEMORY | MFX_IOPATTERN_OUT_VIDEO_MEMORY;
-
-            mfxStatus sts = MFX_STS_TRACE(MFXVideoVPP_QueryIOSurf(session, &tmp_par, req));
-            MFX_CHECK(sts >= MFX_ERR_NONE, sts);
-
-            MFX_CHECK(m_vpp_pool == MFX_VPP_POOL_IN || m_vpp_pool == MFX_VPP_POOL_OUT, MFX_ERR_INVALID_VIDEO_PARAM);
-
-            m_required_num_surf = req[m_vpp_pool].NumFrameSuggested;
-        }
-        break;
-
-        default:
-            MFX_RETURN(MFX_ERR_NOT_IMPLEMENTED);
-        }
-
-        MFX_CHECK(m_cache, MFX_ERR_NOT_INITIALIZED);
-
         auto it = std::find_if(par.ExtParam, par.ExtParam + par.NumExtParam,
             [this](mfxExtBuffer* buffer)
             {
@@ -1038,15 +1060,82 @@ struct surface_cache_controller
                     (m_type != ComponentType::VPP || reinterpret_cast<mfxExtAllocationHints*>(buffer)->VPPPoolType == m_vpp_pool);
             });
 
-        if (it == par.ExtParam + par.NumExtParam)
-            return MFX_ERR_NONE;
+        mfxExtAllocationHints* allocation_hints = nullptr;
 
-        mfxExtAllocationHints* allocation_hints = reinterpret_cast<mfxExtAllocationHints*>(*it);
+        bool optimal_caching_policy_requested = false;
+
+        if (it != par.ExtParam + par.NumExtParam)
+        {
+            allocation_hints = reinterpret_cast<mfxExtAllocationHints*>(*it);
+            optimal_caching_policy_requested = allocation_hints->AllocationPolicy == MFX_ALLOCATION_OPTIMAL;
+        }
+
+        if (optimal_caching_policy_requested)
+        {
+            // Use temporal copy for possible IOPattern adjustment. Hitorically Init doesn't require setting of IOPatter, but QueryIOSurf does.
+            // This function is invoked during Init
+            mfxVideoParam tmp_par = par;
+
+            switch (m_type)
+            {
+            case ComponentType::DECODE:
+            {
+                mfxFrameAllocRequest req = {};
+
+                if (!tmp_par.IOPattern)
+                    tmp_par.IOPattern = MFX_IOPATTERN_OUT_VIDEO_MEMORY;
+
+                mfxStatus sts = MFX_STS_TRACE(MFXVideoDECODE_QueryIOSurf(session, &tmp_par, &req));
+                MFX_CHECK(sts >= MFX_ERR_NONE, sts);
+
+                m_required_num_surf = req.NumFrameSuggested;
+            }
+            break;
+
+            case ComponentType::ENCODE:
+            {
+                mfxFrameAllocRequest req = {};
+
+                if (!tmp_par.IOPattern)
+                    tmp_par.IOPattern = MFX_IOPATTERN_IN_VIDEO_MEMORY;
+
+                mfxStatus sts = MFX_STS_TRACE(MFXVideoENCODE_QueryIOSurf(session, &tmp_par, &req));
+                MFX_CHECK(sts >= MFX_ERR_NONE, sts);
+
+                m_required_num_surf = req.NumFrameSuggested;
+            }
+            break;
+
+            case ComponentType::VPP:
+            {
+                mfxFrameAllocRequest req[2] = {};
+
+                if (!tmp_par.IOPattern)
+                    tmp_par.IOPattern = MFX_IOPATTERN_IN_VIDEO_MEMORY | MFX_IOPATTERN_OUT_VIDEO_MEMORY;
+
+                mfxStatus sts = MFX_STS_TRACE(MFXVideoVPP_QueryIOSurf(session, &tmp_par, req));
+                MFX_CHECK(sts >= MFX_ERR_NONE, sts);
+
+                MFX_CHECK(m_vpp_pool == MFX_VPP_POOL_IN || m_vpp_pool == MFX_VPP_POOL_OUT, MFX_ERR_INVALID_VIDEO_PARAM);
+
+                m_required_num_surf = req[m_vpp_pool].NumFrameSuggested;
+            }
+            break;
+
+            default:
+                MFX_RETURN(MFX_ERR_NOT_IMPLEMENTED);
+            }
+        }
+
+        MFX_CHECK(m_cache, MFX_ERR_NOT_INITIALIZED);
+
+        if (!allocation_hints)
+            return MFX_ERR_NONE;
 
         MFX_SAFE_CALL(CheckAllocationHintsBuffer(*allocation_hints, m_type == ComponentType::VPP));
         MFX_SAFE_CALL(m_cache->SetupPolicy(*allocation_hints));
 
-        if (allocation_hints->AllocationPolicy == MFX_ALLOCATION_OPTIMAL)
+        if (optimal_caching_policy_requested)
         {
             unique_ptr_refcountable<mfxSurfacePoolInterface> scoped_surface_lock(m_cache.get());
             // AddRef here because m_cache is also a smart pointer, so will be decremented twice
@@ -1207,4 +1296,15 @@ inline bool IsAdapt(mfxU32 opt)
 }
 
 
+inline void* AlignedMalloc(size_t size, size_t alignment)
+{
+    return aligned_alloc(alignment, size);
+}
+
+inline void AlignedFree(void* memory)
+{
+    free(memory);
+}
+
+
 #endif // __MFXUTILS_H__
