@@ -19,6 +19,7 @@
 // SOFTWARE.
 
 #include "mfx_common.h"
+#include "mfxcamera.h"
 
 #if defined (MFX_ENABLE_VPP)
 
@@ -93,6 +94,8 @@ namespace MfxHwVideoProcessing
         MFX_FOURCC_Y216      ,
         MFX_FOURCC_Y416      ,
         MFX_FOURCC_BGRP      ,
+        MFX_FOURCC_ABGR16F   ,
+        MFX_FOURCC_I420      ,
     };
 
     typedef enum mfxFormatSupport {
@@ -140,6 +143,22 @@ namespace MfxHwVideoProcessing
         mfxU32 TileId;
     };
 
+    typedef struct _CameraCaps
+    {
+        mfxU32 uBlackLevelCorrection;
+        mfxU32 uHotPixelCheck;
+        mfxU32 uWhiteBalance;
+        mfxU32 uColorCorrectionMatrix;
+        mfxU32 uGammaCorrection;
+        mfxU32 uVignetteCorrection;
+        mfxU32 u3DLUT;
+        mfxU32 uCscYuvRgb;
+        mfxU32 uTotalColorControl;
+        mfxU32 uLensGeomDistCorrection;
+        mfxU32 uPadding;
+        mfxU32 uBayerDenoise;
+    } CameraCaps;
+
     struct mfxVppCaps
     {
         mfxU32 uAdvancedDI;
@@ -153,11 +172,15 @@ namespace MfxHwVideoProcessing
         mfxU32 uProcampFilter;
         mfxU32 uSceneChangeDetection;
 
+        CameraCaps cameraCaps;
+
         mfxU32 uFrameRateConversion;
         mfxU32 uDeinterlacing;
 
         mfxU32 uVideoSignalInfoInOut;
 
+        mfxU32 uHdr10ToneMapping;
+
         mfxU32 uVideoSignalInfo;
         FrcCaps frcCaps;
 
@@ -204,7 +227,7 @@ namespace MfxHwVideoProcessing
             , uDeinterlacing(0)
 
             , uVideoSignalInfoInOut(0)
-
+            , uHdr10ToneMapping(0)
             , uVideoSignalInfo(0)
             , frcCaps()
             , uIStabFilter(0)
@@ -225,6 +248,7 @@ namespace MfxHwVideoProcessing
             , u3DLut(0)
             , uDenoise2Filter(0)
         {
+            memset(&cameraCaps, 0, sizeof(CameraCaps));
         };
     };
 
@@ -295,6 +319,7 @@ namespace MfxHwVideoProcessing
             mfxResourceType           BufferType;
             mfx3DLutMemoryLayout      MemLayout;
             mfx3DLutChannelMapping    ChannelMapping;
+            mfxChannel                Channel[3];
         };
 
     public:
@@ -338,6 +363,33 @@ namespace MfxHwVideoProcessing
                ,bFieldWeavingExt(false)
                ,bFieldSplittingExt(false)
                ,iFieldProcessingMode(0)
+               ,bCameraPipeEnabled(false)
+               ,bCameraPipeControl(0)
+               ,CameraPipeControl()
+               ,bCameraBlackLevelCorrection(false)
+               ,CameraBlackLevel()
+               ,bCameraWhiteBalaceCorrection(false)
+               ,CameraWhiteBalance()
+               ,bCameraHotPixelRemoval(false)
+               ,CameraHotPixel()
+               ,bCCM(false)
+               ,CCMParams()
+               ,bCameraTCC(false)
+               ,CameraTCC()
+               ,bCameraRGBtoYUV(false)
+               ,CameraRGBToYUV()               
+               ,bCameraGammaCorrection(false)
+               ,CameraForwardGammaCorrection()
+               ,bCameraVignetteCorrection(false)
+               ,CameraVignetteCorrection()
+               ,bCameraLensCorrection(false)
+               ,CameraLensCorrection()
+               ,bCamera3DLUT(false)
+               ,Camera3DLUT()
+               ,bCameraPadding(false)
+               ,CameraPadding()
+               ,bCameraBayerDenoise(false)
+               ,CameraBayerDenoise()
                ,rotation(0)
                ,scalingMode(MFX_SCALING_MODE_DEFAULT)
                ,interpolationMethod(MFX_INTERPOLATION_DEFAULT)
@@ -373,6 +425,9 @@ namespace MfxHwVideoProcessing
 
                    m_inVideoSignalInfo  = {};
                    m_outVideoSignalInfo = {};
+
+                   inHDR10MetaData = {};
+                   outHDR10MetaData = {};
             };
 
             bool IsDoNothing()
@@ -397,6 +452,13 @@ namespace MfxHwVideoProcessing
                     bFieldWeaving != false ||
                     bFieldSplittingExt != false ||
                     iFieldProcessingMode != 0 ||
+                    bCameraPipeEnabled != false ||
+                    bCameraBlackLevelCorrection != false ||
+                    bCameraGammaCorrection != false ||
+                    bCameraHotPixelRemoval != false ||
+                    bCameraWhiteBalaceCorrection != false ||
+                    bCCM != false ||
+                    bCameraLensCorrection != false ||
                     rotation != 0 ||
                     scalingMode != MFX_SCALING_MODE_DEFAULT ||
                     mirroring != 0 ||
@@ -473,6 +535,35 @@ namespace MfxHwVideoProcessing
 
         mfxU32         iFieldProcessingMode;
 
+        //  Camera Pipe specific params
+        bool                            bCameraPipeEnabled;
+        bool                            bCameraPipeControl;
+        mfxExtCamPipeControl            CameraPipeControl;
+        bool                            bCameraBlackLevelCorrection;
+        mfxExtCamBlackLevelCorrection   CameraBlackLevel;
+        bool                            bCameraWhiteBalaceCorrection;
+        mfxExtCamWhiteBalance           CameraWhiteBalance;
+        bool                            bCameraHotPixelRemoval;
+        mfxExtCamHotPixelRemoval        CameraHotPixel;
+        bool                            bCCM;
+        mfxExtCamColorCorrection3x3     CCMParams;
+        bool                            bCameraTCC;
+        mfxExtCamTotalColorControl      CameraTCC;
+        bool                            bCameraRGBtoYUV;
+        mfxExtCamCscYuvRgb              CameraRGBToYUV;
+        bool                            bCameraGammaCorrection;
+        mfxExtCamFwdGamma               CameraForwardGammaCorrection;
+        bool                            bCameraVignetteCorrection;
+        mfxExtCamVignetteCorrection     CameraVignetteCorrection;
+        bool                            bCameraLensCorrection;
+        mfxExtCamLensGeomDistCorrection CameraLensCorrection;
+        bool                            bCamera3DLUT;
+        mfxExtCam3DLut                  Camera3DLUT;
+        bool                            bCameraPadding;
+        mfxExtCamPadding                CameraPadding;
+        bool                            bCameraBayerDenoise;
+        mfxExtCamBayerDenoise           CameraBayerDenoise;
+
         int         rotation;
 
         mfxU16      scalingMode;
@@ -508,6 +599,20 @@ namespace MfxHwVideoProcessing
 
         mfxVideoSignalInfo    m_inVideoSignalInfo;
         mfxVideoSignalInfo    m_outVideoSignalInfo;
+
+        struct HDR10MetaData
+        {
+            bool   enabled;
+            mfxU16 displayPrimariesX[3];
+            mfxU16 displayPrimariesY[3];
+            mfxU16 whitePoint[2];
+            mfxU32 maxMasteringLuminance;
+            mfxU32 minMasteringLuminance;
+            mfxU16 maxContentLightLevel;
+            mfxU16 maxFrameAverageLightLevel;
+        };
+        HDR10MetaData inHDR10MetaData;
+        HDR10MetaData outHDR10MetaData;
     };
 
     class DriverVideoProcessing
