@@ -84,6 +84,9 @@ enum eMFXHWType
     MFX_HW_DG2       = MFX_HW_TGL_LP + 8,
 
 
+
+    MFX_HW_MTL       = 0x1700000,
+
 };
 
 enum eMFXGTConfig
@@ -459,22 +462,33 @@ typedef struct {
     /* ADL-P */
     { 0x46A0, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46A1, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
+    { 0x46A2, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46A3, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46A6, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x4626, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
 
     { 0x46B0, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46B1, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
+    { 0x46B2, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46B3, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46A8, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x4628, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
 
     { 0x46C0, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46C1, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
+    { 0x46C2, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46C3, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x46AA, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
     { 0x462A, MFX_HW_ADL_P, MFX_GT2 },//ADL-P
 
+    /* RPL-P */
+    { 0xA7A0, MFX_HW_ADL_P, MFX_GT2 },
+    { 0xA720, MFX_HW_ADL_P, MFX_GT2 },
+    { 0xA7A8, MFX_HW_ADL_P, MFX_GT2 },
+    { 0xA7A1, MFX_HW_ADL_P, MFX_GT2 },
+    { 0xA721, MFX_HW_ADL_P, MFX_GT2 },
+    { 0xA7A9, MFX_HW_ADL_P, MFX_GT2 },
+
     /* ADL-N */
     { 0x46D0, MFX_HW_ADL_N, MFX_GT1 },//ADL-N
     { 0x46D1, MFX_HW_ADL_N, MFX_GT1 },//ADL-N
@@ -511,9 +525,26 @@ typedef struct {
     { 0x56A9, MFX_HW_DG2, MFX_GT4 }, // DG2
     { 0x56B0, MFX_HW_DG2, MFX_GT4 }, // DG2
     { 0x56B1, MFX_HW_DG2, MFX_GT4 }, // DG2
+    { 0x56B2, MFX_HW_DG2, MFX_GT4 }, // DG2
+    { 0x56B3, MFX_HW_DG2, MFX_GT4 }, // DG2
     { 0x56C0, MFX_HW_DG2, MFX_GT4 }, // DG2
     { 0x56C1, MFX_HW_DG2, MFX_GT4 }, // DG2
 
+
+    /* MTL */
+    { 0x7D40, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D50, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D55, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D60, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D70, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D75, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D79, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D76, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D66, MFX_HW_MTL, MFX_GT2 },
+    { 0x7DD5, MFX_HW_MTL, MFX_GT2 },
+    { 0x7D45, MFX_HW_MTL, MFX_GT2 },
+    { 0x7DE0, MFX_HW_MTL, MFX_GT2 },
+
 };
 
 /*
