@@ -97,6 +97,10 @@ struct mfxTraceAlgorithm
 
 static mfxTraceU32      g_OutputMode = MFX_TRACE_OUTPUT_TRASH;
 static mfxTraceU32      g_Level      = MFX_TRACE_LEVEL_DEFAULT;
+mfxTraceU32      EnableEventTrace = 0;
+mfxTraceU32      EventCfg = 0;
+mfxTraceU32      LogConfig = 0;
+int32_t FrameIndex = -1;
 static volatile uint32_t  g_refCounter = 0;
 
 static mfxTraceU32           g_mfxTraceCategoriesNum = 0;
@@ -195,6 +199,39 @@ mfxTraceU32 MFXTrace_GetRegistryParams(void)
 }
 
 
+mfxTraceU32 MFXTrace_GetEnvParams(void)
+{
+    //get VPL TXT log environment variable
+    const char* tracelogChar = std::getenv("VPL_TXT_LOG");
+    char* endPtr = nullptr;
+    if (tracelogChar != nullptr)
+    {
+        LogConfig = std::strtol(tracelogChar, &endPtr, 10);
+        if (LogConfig > 0 && LogConfig <= MFX_TXTLOG_LEVEL_API)
+        {
+            g_OutputMode |= MFX_TRACE_OUTPUT_TEXTLOG;
+            g_Level = LogConfig;
+        }
+    }
+    //Enable event capture according to VPL_EVENT_TRACE
+    const char* g_eventTrace = std::getenv("VPL_EVENT_TRACE");
+    char* endEvent = nullptr;
+    if (g_eventTrace != nullptr)
+    {
+        EnableEventTrace = std::strtol(g_eventTrace, &endEvent, 10);
+        g_OutputMode |= MFX_TRACE_OUTPUT_ETW;
+    }
+    //Capture different info according to VPL_EVENT_TRACE_CFG
+    const char* g_eventCfg = std::getenv("VPL_EVENT_TRACE_CFG");
+    char* endEventCfg = nullptr;
+    if (g_eventCfg != nullptr)
+    {
+        EventCfg = std::strtol(g_eventCfg, &endEventCfg, 16);
+    }
+
+    return 0;
+}
+
 /*------------------------------------------------------------------------------*/
 
 mfxTraceU32 mfx_trace_get_category_index(mfxTraceChar* category, mfxTraceU32& index)
@@ -216,23 +253,56 @@ mfxTraceU32 mfx_trace_get_category_index(mfxTraceChar* category, mfxTraceU32& in
 
 /*------------------------------------------------------------------------------*/
 
-inline bool MFXTrace_IsPrintableCategoryAndLevel(mfxTraceChar* category, mfxTraceU32 level)
+inline bool MFXTrace_IsPrintableCategoryAndLevel(mfxTraceU32 m_OutputInitilized, mfxTraceU32 level)
 {
-    mfxTraceU32 index = 0;
+    bool logFlag = false;
+    if (m_OutputInitilized == MFX_TRACE_OUTPUT_TEXTLOG) {
+#ifndef NDEBUG
+        if (g_Level == MFX_TXTLOG_LEVEL_MAX)
+        {
+            logFlag = true;
+        }
+        else if (g_Level == MFX_TXTLOG_LEVEL_API_AND_INTERNAL)
+        {
+            if (level != MFX_TRACE_LEVEL_API_PARAMS)
+            {
+                logFlag = true;
+            }
+        }
+        else if (g_Level == MFX_TXTLOG_LEVEL_API_AND_PARAMS)
+#else
+        if (g_Level == MFX_TXTLOG_LEVEL_API_AND_PARAMS)
+#endif
+        {
+            if (level == MFX_TRACE_LEVEL_API_PARAMS || level == MFX_TRACE_LEVEL_API)
+            {
+                logFlag = true;
+            }
+        }
+        else if (g_Level == MFX_TXTLOG_LEVEL_API)
+        {
+            if (level == MFX_TRACE_LEVEL_API)
+            {
+                logFlag = true;
+            }
+        }
+    }
+    else if(m_OutputInitilized == MFX_TRACE_OUTPUT_ETW)
+    {
+        if (EventCfg & (1 << MFX_ETWLOG_ALL_LEVEL))
+        {
+            logFlag = true;
+        }
+        else if (EventCfg & (1 << MFX_ETWLOG_API_LEVEL))
+        {
+            if (level == MFX_TRACE_LEVEL_API)
+            {
+                logFlag = true;
+            }
+        }
+    }
 
-    if (!g_OutputMode)
-    {
-        return false;
-    }
-    if (!mfx_trace_get_category_index(category, index))
-    {
-        if (level <= g_mfxTraceCategoriesTable[index].m_level) return true;
-    }
-    else if (!g_mfxTraceCategoriesTable)
-    {
-        if (level <= g_Level) return true;
-    }
-    return false;
+    return logFlag;
 }
 
 /*------------------------------------------------------------------------------*/
@@ -246,12 +316,6 @@ mfxTraceU32 MFXTrace_Init()
 #if defined(MFX_TRACE_ENABLE_TRASH)
     g_OutputMode |= MFX_TRACE_OUTPUT_TRASH;
 #endif
-#if defined(MFX_TRACE_ENABLE_TEXTLOG)
-    g_OutputMode |= MFX_TRACE_OUTPUT_TEXTLOG;
-#endif
-#if defined(MFX_TRACE_ENABLE_STAT)
-    g_OutputMode |= MFX_TRACE_OUTPUT_STAT;
-#endif
 #if defined(MFX_TRACE_ENABLE_STAT)
     g_OutputMode |= MFX_TRACE_OUTPUT_STAT;
 #endif
@@ -278,6 +342,8 @@ mfxTraceU32 MFXTrace_Init()
     }
 
     sts = MFXTrace_GetRegistryParams();
+
+    sts = MFXTrace_GetEnvParams();
     if (!sts)
     {
         output_mode = g_OutputMode;
@@ -361,8 +427,6 @@ mfxTraceU32 MFXTrace_DebugMessage(mfxTraceStaticHandle *static_handle,
                              mfxTraceChar* category, mfxTraceLevel level,
                              const char *message, const char *format, ...)
 {
-    if (!MFXTrace_IsPrintableCategoryAndLevel(category, level)) return 0;
-
     mfxTraceU32 sts = 0, res = 0;
     mfxTraceU32 i = 0;
     va_list args;
@@ -372,6 +436,8 @@ mfxTraceU32 MFXTrace_DebugMessage(mfxTraceStaticHandle *static_handle,
     {
         if (g_OutputMode & g_TraceAlgorithms[i].m_OutputInitilized)
         {
+            if (!MFXTrace_IsPrintableCategoryAndLevel(g_TraceAlgorithms[i].m_OutputInitilized, level)) continue;
+
             res = g_TraceAlgorithms[i].m_vDebugMessageFn(static_handle,
                                                          file_name, line_num,
                                                          function_name,
@@ -393,7 +459,6 @@ mfxTraceU32 MFXTrace_vDebugMessage(mfxTraceStaticHandle *static_handle,
                               const char *message,
                               const char *format, va_list args)
 {
-    if (!MFXTrace_IsPrintableCategoryAndLevel(category, level)) return 0;
 
     mfxTraceU32 sts = 0, res = 0;
     mfxTraceU32 i = 0;
@@ -402,6 +467,8 @@ mfxTraceU32 MFXTrace_vDebugMessage(mfxTraceStaticHandle *static_handle,
     {
         if (g_OutputMode & g_TraceAlgorithms[i].m_OutputInitilized)
         {
+            if (!MFXTrace_IsPrintableCategoryAndLevel(g_TraceAlgorithms[i].m_OutputInitilized, level)) continue;
+
             res = g_TraceAlgorithms[i].m_vDebugMessageFn(static_handle,
                                                          file_name, line_num,
                                                          function_name,
@@ -430,8 +497,6 @@ mfxTraceU32 MFXTrace_BeginTask(mfxTraceStaticHandle *static_handle,
         static_handle->level    = level;
     }
 
-    if (!MFXTrace_IsPrintableCategoryAndLevel(category, level)) return 0;
-
     mfxTraceU32 sts = 0, res = 0;
     mfxTraceU32 i = 0;
 
@@ -439,6 +504,8 @@ mfxTraceU32 MFXTrace_BeginTask(mfxTraceStaticHandle *static_handle,
     {
         if (g_OutputMode & g_TraceAlgorithms[i].m_OutputInitilized)
         {
+            if (!MFXTrace_IsPrintableCategoryAndLevel(g_TraceAlgorithms[i].m_OutputInitilized, level)) continue;
+
             res = g_TraceAlgorithms[i].m_BeginTaskFn(static_handle,
                                                      file_name, line_num,
                                                      function_name,
@@ -467,8 +534,6 @@ mfxTraceU32 MFXTrace_EndTask(mfxTraceStaticHandle *static_handle,
         level    = static_handle->level;
     }
 
-    if (!MFXTrace_IsPrintableCategoryAndLevel(category, level)) return 0;
-
     mfxTraceU32 sts = 0, res = 0;
     mfxTraceU32 i = 0;
 
@@ -476,6 +541,8 @@ mfxTraceU32 MFXTrace_EndTask(mfxTraceStaticHandle *static_handle,
     {
         if (g_OutputMode & g_TraceAlgorithms[i].m_OutputInitilized)
         {
+            if (!MFXTrace_IsPrintableCategoryAndLevel(g_TraceAlgorithms[i].m_OutputInitilized, level)) continue;
+
             res = g_TraceAlgorithms[i].m_EndTaskFn(static_handle, task_handle);
             if (!sts && res) sts = res;
         }
