@@ -1,4 +1,4 @@
-// Copyright (c) 2007-2020 Intel Corporation
+// Copyright (c) 2007-2022 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -103,6 +103,14 @@ static inline mfxU32 ConvertMfxFourccToVAFormat(mfxU32 fourcc)
         return VA_FOURCC_Y216;
     case MFX_FOURCC_Y416:
         return VA_FOURCC_Y416;
+#if defined (DECODE_JPEG_ROTATION)
+    case MFX_FOURCC_YUV422H:
+        return VA_FOURCC_422H;
+    case MFX_FOURCC_YUV422V:
+        return VA_FOURCC_422V;
+#endif
+    case MFX_FOURCC_I420:
+        return VA_FOURCC_I420;
     default:
         assert(!"unsupported fourcc");
         return 0;
@@ -143,7 +151,6 @@ static void FillSurfaceAttrs(std::vector<VASurfaceAttrib> &attrib, unsigned int
         case MFX_FOURCC_YUY2:
             format = VA_RT_FORMAT_YUV422;
             break;
-            break;
         case MFX_FOURCC_A2RGB10:
             format = VA_RT_FORMAT_RGB32_10BPP;
             break;
@@ -209,6 +216,7 @@ static inline bool isFourCCSupported(mfxU32 va_fourcc)
 #ifdef MFX_ENABLE_RGBP
         case VA_FOURCC_RGBP:
 #endif
+        case VA_FOURCC_BGRP:
         case VA_FOURCC_UYVY:
         case VA_FOURCC_P208:
         case VA_FOURCC_P010:
@@ -222,6 +230,11 @@ static inline bool isFourCCSupported(mfxU32 va_fourcc)
         case VA_FOURCC_P016:
         case VA_FOURCC_Y216:
         case VA_FOURCC_Y416:
+#if defined (DECODE_JPEG_ROTATION)
+        case VA_FOURCC_422H:
+        case VA_FOURCC_422V:
+#endif
+        case VA_FOURCC_I420:
             return true;
         default:
             return false;
@@ -482,6 +495,14 @@ static inline mfxU32 SupportedVAfourccToMFXfourcc(mfxU32 va_fourcc)
         return MFX_FOURCC_NV12;
     case MFX_FOURCC_VP8_SEGMAP:
         return MFX_FOURCC_P8;
+    case VA_FOURCC_I420:
+        return MFX_FOURCC_I420;
+    case VA_FOURCC_P012:
+        return VA_FOURCC_P016;
+    case VA_FOURCC_Y212:
+        return VA_FOURCC_Y216;
+    case VA_FOURCC_Y412:
+        return VA_FOURCC_Y416;
     default:
         return va_fourcc;
     }
@@ -508,6 +529,12 @@ mfxStatus mfxDefaultAllocatorVAAPI::SetFrameData(const VAImage &va_image, mfxU32
         frame_data.U = p_buffer + va_image.offsets[2];
         break;
 
+    case VA_FOURCC_I420:
+        frame_data.Y = p_buffer + va_image.offsets[0];
+        frame_data.U = p_buffer + va_image.offsets[1];
+        frame_data.V = p_buffer + va_image.offsets[2];
+        break;
+
     case VA_FOURCC_YUY2:
         frame_data.Y = p_buffer + va_image.offsets[0];
         frame_data.U = frame_data.Y + 1;
@@ -549,6 +576,11 @@ mfxStatus mfxDefaultAllocatorVAAPI::SetFrameData(const VAImage &va_image, mfxU32
         frame_data.B = p_buffer + va_image.offsets[2];
         break;
 #endif
+    case VA_FOURCC_BGRP:
+        frame_data.B = p_buffer + va_image.offsets[0];
+        frame_data.G = p_buffer + va_image.offsets[1];
+        frame_data.R = p_buffer + va_image.offsets[2];
+        break;
     case VA_FOURCC_ABGR:
         frame_data.R = p_buffer + va_image.offsets[0];
         frame_data.G = frame_data.R + 1;
@@ -561,6 +593,7 @@ mfxStatus mfxDefaultAllocatorVAAPI::SetFrameData(const VAImage &va_image, mfxU32
         break;
 
     case VA_FOURCC_P010:
+    case VA_FOURCC_P012:
     case VA_FOURCC_P016:
         frame_data.Y = p_buffer + va_image.offsets[0];
         frame_data.U = p_buffer + va_image.offsets[1];
@@ -575,6 +608,7 @@ mfxStatus mfxDefaultAllocatorVAAPI::SetFrameData(const VAImage &va_image, mfxU32
         break;
 
     case VA_FOURCC_Y210:
+    case VA_FOURCC_Y212:
     case VA_FOURCC_Y216:
         frame_data.Y16 = (mfxU16 *) (p_buffer + va_image.offsets[0]);
         frame_data.U16 = frame_data.Y16 + 1;
@@ -586,6 +620,7 @@ mfxStatus mfxDefaultAllocatorVAAPI::SetFrameData(const VAImage &va_image, mfxU32
         frame_data.Y410 = (mfxY410*)(p_buffer + va_image.offsets[0]);
         break;
 
+    case VA_FOURCC_Y412:
     case VA_FOURCC_Y416:
         frame_data.U16 = (mfxU16 *) (p_buffer + va_image.offsets[0]);
         frame_data.Y16 = frame_data.U16 + 1;
