@@ -70,6 +70,7 @@ mfxStatus MFXVideoCORE_QueryPlatform(mfxSession session, mfxPlatform* platform)
 
 mfxStatus MFXMemory_GetSurfaceForDecode(mfxSession session, mfxFrameSurface1** output_surf)
 {
+    MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_API, __FUNCTION__);
     MFX_CHECK_NULL_PTR1(output_surf);
     MFX_CHECK_HDL(session);
     MFX_CHECK(session->m_pCORE.get(), MFX_ERR_NOT_INITIALIZED);
@@ -695,7 +696,7 @@ bool CommonCORE::IsExternalFrameAllocator() const
     return m_bSetExtFrameAlloc;
 }
 
-mfxStatus CommonCORE::DoFastCopyWrapper(mfxFrameSurface1 *pDst, mfxU16 dstMemType, mfxFrameSurface1 *pSrc, mfxU16 srcMemType, bool)
+mfxStatus CommonCORE::DoFastCopyWrapper(mfxFrameSurface1 *pDst, mfxU16 dstMemType, mfxFrameSurface1 *pSrc, mfxU16 srcMemType, mfxU32)
 {
     mfxStatus sts = MFX_ERR_NONE;
 
@@ -939,6 +940,12 @@ mfxStatus CoreDoSWFastCopy(mfxFrameSurface1 & dst, const mfxFrameSurface1 & src,
 
     switch (dst.Info.FourCC)
     {
+    case MFX_FOURCC_R16:
+        roi.width *= 2;
+
+        MFX_SAFE_CALL(FastCopy::Copy(dst.Data.R, dstPitch, src.Data.R, srcPitch, roi, copyFlag));
+        return MFX_ERR_NONE;
+
     case MFX_FOURCC_P010:
     case MFX_FOURCC_P016:
 
@@ -1002,6 +1009,20 @@ mfxStatus CoreDoSWFastCopy(mfxFrameSurface1 & dst, const mfxFrameSurface1 & src,
 
         return FastCopy::Copy(dst.Data.V, dstPitch, src.Data.V, srcPitch, roi, copyFlag);
 
+    case MFX_FOURCC_I420:
+
+        MFX_SAFE_CALL(FastCopy::Copy(dst.Data.Y, dstPitch, src.Data.Y, srcPitch, roi, copyFlag));
+
+        roi.width  >>= 1;
+        roi.height >>= 1;
+
+        srcPitch >>= 1;
+        dstPitch >>= 1;
+
+        MFX_SAFE_CALL(FastCopy::Copy(dst.Data.U, dstPitch, src.Data.U, srcPitch, roi, copyFlag));
+
+        return FastCopy::Copy(dst.Data.V, dstPitch, src.Data.V, srcPitch, roi, copyFlag);
+
     case MFX_FOURCC_UYVY:
         roi.width *= 2;
 
@@ -1133,13 +1154,23 @@ mfxStatus CoreDoSWFastCopy(mfxFrameSurface1 & dst, const mfxFrameSurface1 & src,
     }
     case MFX_FOURCC_P8:
         return FastCopy::Copy(dst.Data.Y, dstPitch, src.Data.Y, srcPitch, roi, copyFlag);
+    case MFX_FOURCC_ABGR16F:
+    {
+        MFX_CHECK_NULL_PTR1(src.Data.ABGRFP16);
+        mfxU8* ptrSrc = (mfxU8*)src.Data.ABGRFP16;
+        mfxU8* ptrDst = (mfxU8*)dst.Data.ABGRFP16;
+
+        roi.width *= 8;
+
+        return FastCopy::Copy(ptrDst, dstPitch, ptrSrc, srcPitch, roi, copyFlag);
+    }
 
     default:
         MFX_RETURN(MFX_ERR_UNSUPPORTED);
     }
 }
 
-mfxStatus CommonCORE::DoFastCopyExtended(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, bool)
+mfxStatus CommonCORE::DoFastCopyExtended(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, mfxU32)
 {
     // up mutex
     UMC::AutomaticUMCMutex guard(m_guard);
@@ -1537,7 +1568,7 @@ mfxStatus CommonCORE_VPL::SwitchMemidInSurface(mfxFrameSurface1 & surf, mfxHDLPa
 }
 
 
-mfxStatus CommonCORE_VPL::DoFastCopyWrapper(mfxFrameSurface1 *pDst, mfxU16 dstMemType, mfxFrameSurface1 *pSrc, mfxU16 srcMemType, bool)
+mfxStatus CommonCORE_VPL::DoFastCopyWrapper(mfxFrameSurface1 *pDst, mfxU16 dstMemType, mfxFrameSurface1 *pSrc, mfxU16 srcMemType, mfxU32)
 {
     MFX_CHECK_NULL_PTR2(pSrc, pDst);
 
@@ -1569,7 +1600,7 @@ mfxStatus CommonCORE_VPL::DoFastCopyWrapper(mfxFrameSurface1 *pDst, mfxU16 dstMe
     return dst_surf_lock.unlock();
 }
 
-mfxStatus CommonCORE_VPL::DoFastCopyExtended(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, bool)
+mfxStatus CommonCORE_VPL::DoFastCopyExtended(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, mfxU32)
 {
     UMC::AutomaticUMCMutex guard(m_guard);
 
