@@ -34,7 +34,11 @@
 
 #include "libmfx_core_hw.h"
 
+#ifdef MFX_ENABLE_EXT
 #include "cm_mem_copy.h"
+#else
+#include "cm_mem_copy_stub.h"
+#endif
 
 #include <sys/ioctl.h>
 
@@ -205,6 +209,14 @@ eMFXHWType VAAPIVideoCORE_T<Base>::GetHWType()
     return m_HWType;
 }
 
+template <class Base>
+mfxU16 VAAPIVideoCORE_T<Base>::GetHWDeviceId()
+{
+    std::ignore = MFX_STS_TRACE(this->CheckOrInitDisplay());
+
+    return this->m_deviceId;
+}
+
 template <class Base>
 mfxStatus VAAPIVideoCORE_T<Base>::GetHandle(
     mfxHandleType type,
@@ -271,13 +283,20 @@ bool VAAPIVideoCORE_T<Base>::IsCmSupported()
     return GetHWType() < MFX_HW_DG2;
 }
 
+template <class Base>
+bool VAAPIVideoCORE_T<Base>::IsCmCopyEnabledByDefault()
+{
+    // For Linux by default CM copy is ON on RKL/ADL
+    return IsCmSupported() && GetHWType() != MFX_HW_DG1 && GetHWType() != MFX_HW_TGL_LP;
+}
+
 template <class Base>
 mfxStatus VAAPIVideoCORE_T<Base>::TryInitializeCm()
 {
     if (m_pCmCopy)
         return MFX_ERR_NONE;
 
-    // Return immidiately if user requested to turn OFF GPU copy
+    // Return immediately if user requested to turn OFF GPU copy
     if (m_ForcedCmState == MFX_GPUCOPY_OFF)
     {
         return MFX_ERR_NONE;
@@ -289,7 +308,7 @@ mfxStatus VAAPIVideoCORE_T<Base>::TryInitializeCm()
 
     MFX_CHECK_NULL_PTR1(tmp_cm->GetCmDevice(*m_p_display_wrapper));
 
-    MFX_SAFE_CALL(tmp_cm->Initialize(GetHWType()));
+    MFX_SAFE_CALL(tmp_cm->Initialize(GetHWDeviceId(), GetHWType()));
 
     m_pCmCopy = std::move(tmp_cm);
 
@@ -584,7 +603,7 @@ mfxStatus VAAPIVideoCORE_T<Base>::DoFastCopyWrapper(
     mfxU16 dstMemType,
     mfxFrameSurface1* pSrc,
     mfxU16 srcMemType,
-    bool canUseGpuCopy)
+    mfxU32 gpuCopyMode)
 {
     MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_HOTSPOTS, "VAAPIVideoCORE_T<Base>::DoFastCopyWrapper");
     mfxStatus sts;
@@ -712,7 +731,7 @@ mfxStatus VAAPIVideoCORE_T<Base>::DoFastCopyWrapper(
         }
     }
 
-    mfxStatus fcSts = DoFastCopyExtended(&dstTempSurface, &srcTempSurface, canUseGpuCopy);
+    mfxStatus fcSts = DoFastCopyExtended(&dstTempSurface, &srcTempSurface, gpuCopyMode);
 
     if (MFX_ERR_DEVICE_FAILED == fcSts && 0 != dstTempSurface.Data.Corrupted)
     {
@@ -761,7 +780,7 @@ template <class Base>
 mfxStatus VAAPIVideoCORE_T<Base>::DoFastCopyExtended(
     mfxFrameSurface1* pDst,
     mfxFrameSurface1* pSrc,
-    bool canUseGpuCopy)
+    mfxU32 gpuCopyMode)
 {
     mfxStatus sts;
     mfxU8* srcPtr;
@@ -790,8 +809,8 @@ mfxStatus VAAPIVideoCORE_T<Base>::DoFastCopyExtended(
         return MFX_ERR_UNDEFINED_BEHAVIOR;
     }
 
-    // For Linux by default CM copy is OFF
-    bool canUseCMCopy = canUseGpuCopy && m_pCmCopy && m_ForcedCmState == MFX_GPUCOPY_ON && CmCopyWrapper::CanUseCmCopy(pDst, pSrc);
+    // For Linux, if CM copy is forced to be used, or if choose to use default copy method and CM copy is enabled by default, use CM copy.
+    bool canUseCMCopy = (gpuCopyMode & MFX_COPY_USE_CM) && m_pCmCopy && (m_ForcedCmState == MFX_GPUCOPY_ON || (m_ForcedCmState == MFX_GPUCOPY_DEFAULT && IsCmCopyEnabledByDefault())) && CmCopyWrapper::CanUseCmCopy(pDst, pSrc);
 
     if (NULL != pSrc->Data.MemId && NULL != pDst->Data.MemId)
     {
@@ -943,7 +962,7 @@ mfxStatus VAAPIVideoCORE_T<Base>::DoFastCopyExtended(
 
     return MFX_ERR_NONE;
 
-} // mfxStatus VAAPIVideoCORE_T<Base>::DoFastCopyExtended(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, bool canUseGpuCopy)
+} // mfxStatus VAAPIVideoCORE_T<Base>::DoFastCopyExtended(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, bool gpuCopyMode)
 
 // On linux/android specific function!
 // correct work since libva 1.2 (libva 2.2.1.pre1)
@@ -952,8 +971,8 @@ template <class Base>
 mfxStatus VAAPIVideoCORE_T<Base>::IsGuidSupported(const GUID guid,
                                          mfxVideoParam *par, bool /* isEncoder */)
 {
-    MFX_CHECK(par, MFX_WRN_PARTIAL_ACCELERATION);
-    MFX_CHECK(!IsMVCProfile(par->mfx.CodecProfile), MFX_WRN_PARTIAL_ACCELERATION);
+    MFX_CHECK(par, MFX_ERR_UNSUPPORTED);
+    MFX_CHECK(!IsMVCProfile(par->mfx.CodecProfile), MFX_ERR_UNSUPPORTED);
 
     MFX_SAFE_CALL(this->CheckOrInitDisplay());
 
@@ -1034,10 +1053,10 @@ mfxStatus VAAPIVideoCORE_T<Base>::IsGuidSupported(const GUID guid,
     case MFX_CODEC_VP9:
         break;
     case MFX_CODEC_HEVC:
-        MFX_CHECK(par->mfx.FrameInfo.Width <= 8192 && par->mfx.FrameInfo.Height <= 8192, MFX_WRN_PARTIAL_ACCELERATION);
+        MFX_CHECK(par->mfx.FrameInfo.Width <= 8192 && par->mfx.FrameInfo.Height <= 8192, MFX_ERR_UNSUPPORTED);
         break;
     case MFX_CODEC_MPEG2: //MPEG2 decoder doesn't support resolution bigger than 2K
-        MFX_CHECK(par->mfx.FrameInfo.Width <= 2048 && par->mfx.FrameInfo.Height <= 2048, MFX_WRN_PARTIAL_ACCELERATION);
+        MFX_CHECK(par->mfx.FrameInfo.Width <= 2048 && par->mfx.FrameInfo.Height <= 2048, MFX_ERR_UNSUPPORTED);
         break;
     case MFX_CODEC_JPEG:
         MFX_CHECK(par->mfx.FrameInfo.Width <= 8192 && par->mfx.FrameInfo.Height <= 8192, MFX_WRN_PARTIAL_ACCELERATION);
@@ -1050,7 +1069,7 @@ mfxStatus VAAPIVideoCORE_T<Base>::IsGuidSupported(const GUID guid,
 
     MFX_CHECK(MFX_CODEC_JPEG == par->mfx.CodecId || MFX_CODEC_HEVC == par->mfx.CodecId ||
         (par->mfx.FrameInfo.Width <= 4096 && par->mfx.FrameInfo.Height <= 4096),
-        MFX_WRN_PARTIAL_ACCELERATION
+        MFX_ERR_UNSUPPORTED
     );
 
     return MFX_ERR_NONE;
@@ -1205,7 +1224,7 @@ VAAPIVideoCORE_VPL::DoFastCopyWrapper(
     mfxU16 dstMemType,
     mfxFrameSurface1* pSrc,
     mfxU16 srcMemType,
-    bool canUseGpuCopy)
+    mfxU32 gpuCopyMode)
 {
     MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_HOTSPOTS, "VAAPIVideoCORE_VPL::DoFastCopyWrapper");
 
@@ -1249,7 +1268,7 @@ VAAPIVideoCORE_VPL::DoFastCopyWrapper(
         dstTempSurface.Data.MemId = 0;
     }
 
-    sts = DoFastCopyExtended(&dstTempSurface, &srcTempSurface, canUseGpuCopy);
+    sts = DoFastCopyExtended(&dstTempSurface, &srcTempSurface, gpuCopyMode);
     MFX_CHECK_STS(sts);
 
     sts = src_surf_lock.unlock();
@@ -1262,7 +1281,7 @@ mfxStatus
 VAAPIVideoCORE_VPL::DoFastCopyExtended(
     mfxFrameSurface1* pDst,
     mfxFrameSurface1* pSrc,
-    bool canUseGpuCopy)
+    mfxU32 gpuCopyMode)
 {
     MFX_CHECK_NULL_PTR2(pDst, pSrc);
 
@@ -1283,11 +1302,12 @@ VAAPIVideoCORE_VPL::DoFastCopyExtended(
     // check that region of interest is valid
     MFX_CHECK(roi.width && roi.height, MFX_ERR_UNDEFINED_BEHAVIOR);
 
-    // For Linux by default CM copy is OFF
-    bool canUseCMCopy = canUseGpuCopy && m_pCmCopy && m_ForcedCmState == MFX_GPUCOPY_ON && CmCopyWrapper::CanUseCmCopy(pDst, pSrc);
+    // For Linux, if CM copy is forced to be used, or if choose to use default copy method and CM copy is enabled by default, use CM copy.
+    bool canUseCMCopy = (gpuCopyMode & MFX_COPY_USE_CM) && m_pCmCopy && (m_ForcedCmState == MFX_GPUCOPY_ON || (m_ForcedCmState == MFX_GPUCOPY_DEFAULT && IsCmCopyEnabledByDefault())) && CmCopyWrapper::CanUseCmCopy(pDst, pSrc);
 
     if (NULL != pSrc->Data.MemId && NULL != pDst->Data.MemId)
     {
+        // At First try to copy with CM
         if (canUseCMCopy)
         {
             // If CM copy failed, fallback to VA copy
@@ -1295,6 +1315,7 @@ VAAPIVideoCORE_VPL::DoFastCopyExtended(
             // Remove CM adapter in case of failed copy
             this->SetCmCopyStatus(false);
         }
+        // Fallback to VA copy in case of failed CM copy
 
         VASurfaceID *va_surf_src = (VASurfaceID*)(((mfxHDLPair *)pSrc->Data.MemId)->first);
         VASurfaceID *va_surf_dst = (VASurfaceID*)(((mfxHDLPair *)pDst->Data.MemId)->first);
@@ -1319,6 +1340,7 @@ VAAPIVideoCORE_VPL::DoFastCopyExtended(
 
     if (NULL != pSrc->Data.MemId && NULL != dstPtr)
     {
+        // At First try to copy with CM
         if (canUseCMCopy)
         {
             // If CM copy failed, fallback to VA copy
@@ -1326,6 +1348,7 @@ VAAPIVideoCORE_VPL::DoFastCopyExtended(
             // Remove CM adapter in case of failed copy
             this->SetCmCopyStatus(false);
         }
+        // Fallback to SW copy in case of failed CM copy
 
         VASurfaceID *va_surface = (VASurfaceID*)(((mfxHDLPair *)pSrc->Data.MemId)->first);
         MFX_CHECK_HDL(va_surface);
@@ -1369,6 +1392,7 @@ VAAPIVideoCORE_VPL::DoFastCopyExtended(
 
     if (NULL != srcPtr && NULL != pDst->Data.MemId)
     {
+        // At First try to copy with CM
         if (canUseCMCopy)
         {
             // If CM copy failed, fallback to VA copy
@@ -1376,6 +1400,7 @@ VAAPIVideoCORE_VPL::DoFastCopyExtended(
             // Remove CM adapter in case of failed copy
             this->SetCmCopyStatus(false);
         }
+        // Fallback to SW copy in case of failed CM copy
 
         VASurfaceID *va_surface = (VASurfaceID*)(((mfxHDLPair *)pDst->Data.MemId)->first);
         MFX_CHECK_HDL(va_surface);
