@@ -31,6 +31,8 @@
 
 #include <algorithm>
 
+#include "mfx_unified_av1d_logging.h"
+
 namespace UMC_AV1_DECODER
 {
     AV1Decoder::AV1Decoder()
@@ -128,7 +130,8 @@ namespace UMC_AV1_DECODER
 
                 OBUInfo obuInfo;
                 bs.ReadOBUInfo(obuInfo);
-                assert(CheckOBUType(obuInfo.header.obu_type)); // [clean up] Need to remove assert once decoder code is stabilized
+                if (obuInfo.header.obu_type > OBU_PADDING)
+                    return UMC::UMC_ERR_INVALID_PARAMS;
 
                 if (obuInfo.header.obu_type == OBU_SEQUENCE_HEADER)
                 {
@@ -140,7 +143,9 @@ namespace UMC_AV1_DECODER
                         return UMC::UMC_OK;
                 }
 
-                in->MoveDataPointer(static_cast<int32_t>(obuInfo.size));
+                if(in->MoveDataPointer(static_cast<int32_t>(obuInfo.size)) != UMC::UMC_OK)
+                    return UMC::UMC_ERR_NOT_ENOUGH_DATA;
+
             }
             catch (av1_exception const& e)
             {
@@ -343,7 +348,7 @@ namespace UMC_AV1_DECODER
                 {
                     if ((Repeat_H.refresh_frame_flags >> i) & 1)
                     {
-                        if (!frameDPB.empty() && frameDPB[i])
+                        if (!frameDPB.empty() && frameDPB[i] && frameDPB[i]->GetRefCounter())
                            frameDPB[i]->DecrementReference();
 
                         frameDPB[i] = const_cast<AV1DecoderFrame*>(pFrame);
@@ -453,6 +458,9 @@ namespace UMC_AV1_DECODER
         case OBU_REDUNDANT_FRAME_HEADER:
         case OBU_TILE_GROUP:
         case OBU_TILE_LIST:
+        case OBU_PADDING:
+        case OBU_SEQUENCE_HEADER:
+        case OBU_METADATA:
             return false;
         default:
             return true;
@@ -546,10 +554,16 @@ namespace UMC_AV1_DECODER
             Curr_temp = Curr;
         }
 
+        if (!updated_refs.empty())
+        {
+            TRACE_BUFFER_EVENT(MFX_TRACE_API_AV1_DPBPARAMETER_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_DPB_INFO,
+                updated_refs, AV1DecodeDpbInfo, DPBINFO_AV1D);
+        }
+
         bool gotFullFrame = false;
         bool repeatedFrame = false;
         bool skipFrame = false;
-        bool anchor_decode = params.lst_mode && anchor_frames_count < params.anchors_num;
+        bool anchor_decode = params.lst_mode && m_isAnchor;
         bool firstSubmission = false;
  
         AV1DecoderFrame* pCurrFrame = nullptr;
@@ -573,6 +587,7 @@ namespace UMC_AV1_DECODER
             TileLayout layout;
 
             UMC::MediaData tmp = *in; // use local copy of [in] for OBU header parsing to not move data pointer in original [in] prematurely
+            OldPreFrame_id = PreFrame_id;
 
             while (tmp.GetDataSize() >= MINIMAL_DATA_SIZE && gotFullFrame == false && repeatedFrame == false)
             {
@@ -585,7 +600,9 @@ namespace UMC_AV1_DECODER
                 bs.ReadOBUInfo(obuInfo);
                 const AV1_OBU_TYPE obuType = obuInfo.header.obu_type;
 
-                assert(CheckOBUType(obuType)); // [clean up] Need to remove assert once decoder code is stabilized
+                if (obuInfo.header.obu_type > OBU_PADDING)
+                    return UMC::UMC_ERR_INVALID_PARAMS;
+
                 if (tmp.GetDataSize() < obuInfo.size) // not enough data left in the buffer to hold full OBU unit
                     break;
 
@@ -619,12 +636,6 @@ namespace UMC_AV1_DECODER
                     break;
                 }
                 case OBU_TILE_LIST:
-                    // This OBU type is purely for the large scale tile decoding mode.
-                    if (anchor_frames_count < params.anchors_num)
-                    {
-                        throw av1_exception(UMC::UMC_ERR_INVALID_STREAM);
-                    }
-
                     // a ext-tile sequence composition: a sequence header OBU, followed by
                     // a number of OBUs that together constitue one or more coded anchors frames, followed by
                     // a frame header OBU, followed by
@@ -677,7 +688,7 @@ namespace UMC_AV1_DECODER
                 case OBU_FRAME:
                     if (!sequence_header.get())
                         break; // bypass frame header if there is no active seq header
-                    if (!gotFrameHeader)
+                    if (!gotFrameHeader && !updated_refs.empty())
                     {
                         // we read only first entry of uncompressed header in the frame
                         // each subsequent copy of uncompressed header (i.e. OBU_REDUNDANT_FRAME_HEADER) must be exact copy of first entry by AV1 spec
@@ -746,6 +757,13 @@ namespace UMC_AV1_DECODER
                 tmp.MoveDataPointer(static_cast<int32_t>(obuInfo.size));
             }
 
+            // For small decbufsize, cur bitstream may not contain any tile, then deocder will read more data and do header parse again.
+            // So we use OldPreFrame_id to update PreFrame_id to avoid the Frame_id check fail in ReadUncompressedHeader.
+            if (layout.empty() && !gotFullFrame)
+                PreFrame_id = OldPreFrame_id;
+            else if (gotFullFrame)
+                OldPreFrame_id = PreFrame_id;
+
             if (!params.lst_mode || !anchor_decode)
             {
                 if (!HaveTilesToSubmit(pFrameInProgress, layout) && !repeatedFrame)
@@ -758,6 +776,9 @@ namespace UMC_AV1_DECODER
                 }
             }
 
+            if (!sequence_header.get())
+                return UMC::UMC_ERR_NOT_ENOUGH_DATA;
+
             if (anchor_decode)
             {
                 // For anchor frame decoding in ext-tile mode, no need to update DPB since they could be treated as
@@ -773,8 +794,11 @@ namespace UMC_AV1_DECODER
             }
             else
             {
-                pCurrFrame = pFrameInProgress ?
-                    pFrameInProgress : StartFrame(fh, updated_refs, pPrevFrame);
+                if (!updated_refs.empty())
+                {
+                    pCurrFrame = pFrameInProgress ?
+                        pFrameInProgress : StartFrame(fh, updated_refs, pPrevFrame);
+                }
  
                 CompleteDecodedFrames(fh, pCurrFrame, pPrevFrame);
                 CalcFrameTime(pCurrFrame);
@@ -800,7 +824,7 @@ namespace UMC_AV1_DECODER
             if (repeatedFrame)
             {
                 pCurrFrame->ShowAsExisting(true);
-                if (anchor_decode)
+                if (anchor_decode && !updated_refs.empty())
                 {
                     m_anchor_frame_mem_ids[anchor_frames_count] = updated_refs[pCurrFrame->GetFrameHeader().frame_to_show_map_idx]->m_index;
                     RegisterAnchorFrame(anchor_frames_count++);
@@ -858,7 +882,7 @@ namespace UMC_AV1_DECODER
             }
 
             if (anchor_decode)
-                m_anchor_frame_mem_ids[anchor_frames_count] = pCurrFrame->m_index;
+                m_anchor_frame_mem_ids[m_specified_anchor_Idx] = pCurrFrame->m_index;
         }
 
         UMC::Status umcRes = UMC::UMC_OK;
@@ -890,6 +914,27 @@ namespace UMC_AV1_DECODER
         return UMC::UMC_OK;
     }
 
+    bool AV1Decoder::IsFreeSlotInDPB(){
+        int emptyCounter = 0;
+        int readyCounter = 0;
+        for(AV1DecoderFrame* fr : dpb){
+            if(fr->Empty()){
+                emptyCounter++;
+            }
+            if(fr->Outputted() && fr->Displayed() && !fr->RefValid()){
+                readyCounter++;
+            }
+        }
+
+        //this is temporary workaround for parallel encoding use case, that uses
+        //big value of async depth >= 20 and as a result big DPB
+        if(emptyCounter == 2 && readyCounter == 0 && dpb.size() > 20){
+            return false;
+        }
+
+        return true;
+    }
+
     AV1DecoderFrame* AV1Decoder::FindFrameByMemID(UMC::FrameMemID id)
     {
         return FindFrame(
