@@ -33,6 +33,7 @@
 
 #include "umc_av1_frame.h"
 #include "umc_av1_va_packer_vaapi.h"
+#include "mfx_unified_av1d_logging.h"
 
 using namespace UMC;
 
@@ -93,7 +94,7 @@ namespace UMC_AV1_DECODER
             return;
 
         UMC::UMCVACompBuffer* compBufBs = nullptr;
-        uint8_t* const bitstreamData = (uint8_t *)m_va->GetCompBuffer(VASliceDataBufferType, &compBufBs, CalcSizeOfTileSets(tileSets));
+        uint8_t* const bitstreamData = (uint8_t *)m_va->GetCompBuffer(VASliceDataBufferType, &compBufBs, CalcSizeOfTileSets(tileSets), tileSets.size());
         if (!bitstreamData || !compBufBs)
             throw av1_exception(MFX_ERR_MEMORY_ALLOC);
 
@@ -126,7 +127,7 @@ namespace UMC_AV1_DECODER
 
         UMCVACompBuffer* compBufTile = nullptr;
         const int32_t tileControlInfoSize = static_cast<int32_t>(sizeof(VASliceParameterBufferAV1) * tileControlParams.size());
-        VASliceParameterBufferAV1 *tileControlParam = (VASliceParameterBufferAV1*)m_va->GetCompBuffer(VASliceParameterBufferType, &compBufTile, tileControlInfoSize);
+        VASliceParameterBufferAV1 *tileControlParam = (VASliceParameterBufferAV1*)m_va->GetCompBuffer(VASliceParameterBufferType, &compBufTile, tileControlInfoSize, tileSets.size());
         if (!tileControlParam || !compBufTile || (compBufTile->GetBufferSize() < tileControlInfoSize))
             throw av1_exception(MFX_ERR_MEMORY_ALLOC);
 
@@ -407,6 +408,9 @@ namespace UMC_AV1_DECODER
         }
 
         picParam.context_update_tile_id = (uint16_t)info.tile_info.context_update_tile_id;
+
+        TRACE_BUFFER_EVENT(VA_TRACE_API_AV1_PICTUREPARAMETER_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_PICPARAM,
+            picParam, AV1DecodePicparam, PICTUREPARAM_AV1D);
     }
 
     void PackerVA::PackTileControlParams(VASliceParameterBufferAV1& tileControlParam, TileLocation const& loc)
@@ -435,6 +439,9 @@ namespace UMC_AV1_DECODER
             tileControlParam.anchor_frame_idx = loc.anchorFrameIdx;
             tileControlParam.tile_idx_in_tile_list = loc.tileIdxInTileList;
         }
+
+        TRACE_BUFFER_EVENT(VA_TRACE_API_AV1_TILECONTROLPARAMETER_TASK, EVENT_TYPE_INFO, TR_KEY_DECODE_TILEPARAM,
+            tileControlParam, AV1DecodeTileControlparam, TILECONTROLPARAMS_AV1D);
     }
 
 } // namespace UMC_AV1_DECODER
