@@ -735,7 +735,7 @@ VACompBuffer* LinuxVideoAccelerator::GetCompBufferHW(int32_t type, int32_t size,
                 va_num_elements = size/sizeof(VASliceParameterBufferMPEG2);
                 break;
             case UMC::VA_H264:
-                if (m_bH264ShortSlice)
+                if (m_bShortSlice)
                 {
                     va_size         = sizeof(VASliceParameterBufferBase);
                     va_num_elements = size/sizeof(VASliceParameterBufferBase);
@@ -828,7 +828,7 @@ LinuxVideoAccelerator::Execute()
             pCompBuf = m_pCompBuffers[i];
             id = pCompBuf->GetID();
 
-            if (!m_bH264ShortSlice)
+            if (!m_bShortSlice)
             {
                 if (pCompBuf->GetType() == VASliceParameterBufferType)
                 {
@@ -1031,7 +1031,6 @@ Status LinuxVideoAccelerator::SyncTask(int32_t FrameBufIndex, void *surfCorrupti
 {
     Status umcRes = 0;
     MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_HOTSPOTS, "Decode DDIWaitTaskSync");
-    PERF_EVENT(MFX_TRACE_HOTSPOT_DDI_WAIT_TASK_SYNC, 0, make_event_data(), [&]() { return make_event_data(umcRes);});
 
     if (FrameBufIndex < 0)
         return UMC_ERR_INVALID_PARAMS;
@@ -1046,6 +1045,10 @@ Status LinuxVideoAccelerator::SyncTask(int32_t FrameBufIndex, void *surfCorrupti
         MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_EXTCALL, "vaSyncSurface");
         va_sts = vaSyncSurface(m_dpy, *surface);
     }
+    if (EnableEventTrace)
+    {
+        TRACE_EVENT(MFX_TRACE_HOTSPOT_DDI_WAIT_TASK_SYNC, EVENT_TYPE_INFO, 0, make_event_data(FrameBufIndex, 0, va_sts));
+    }
     if (VA_STATUS_ERROR_DECODING_ERROR == va_sts)
     {
         if (surfCorruption) *(uint16_t*)surfCorruption = GetDecodingError();
